#include <QtGui>
#include <QMainWindow>

#include <mainwindow.h>
#include <defines.h>
#include <QStringList>
#include <QDebug>
#include <QFileDialog>
#include <QString>
#include <QDesktopServices>

#include <windowcapture.h>
#include <windowtransactions.h>

MainWindow::MainWindow()
   {
   setupUi(this);

	mdiArea = new QMdiArea;
	mdiArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	mdiArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	setCentralWidget(mdiArea);

   showMaximized();
	setWindowTitle(program_name);

	voipManager = new VoIPFixManager();
	voipManager->startManager();

   deviceOpened = false;
   fileOpened = false;

   windowTrans = NULL;
   windowCap = NULL;
	winGraph = NULL;
	winPackets = NULL;
	winReport = NULL;
	winPlay = NULL;
	winFilter = NULL;
   winStatistics = NULL;
   winAbout = NULL;

   connect(actionSair, SIGNAL(triggered()), this, SLOT(actClose()));
   connect(actionIniciar, SIGNAL(triggered()), this, SLOT(actStartCapture()));
   connect(actionParar, SIGNAL(triggered()), this, SLOT(actStopCapture()));
   connect(actionAbrir, SIGNAL(triggered()), this, SLOT(actOpenFile()));
   connect(actionAbrir_Sequencial, SIGNAL(triggered()), this, SLOT(actOpenSequentialFiles()));
   connect(actionSalvar, SIGNAL(triggered()), this, SLOT(actSaveFile()));
   connect(actionAbout, SIGNAL(triggered()), this, SLOT(actAbout()));
   connect(actionManual, SIGNAL(triggered()), this, SLOT(actManual()));
   connect(actionWebsite, SIGNAL(triggered()), this, SLOT(actWebsite()));

	actionEstatisticas->setEnabled(false);
	actionFiltro->setEnabled(false);

   actionAbrir->setEnabled(true);
   actionAbrir_Sequencial->setEnabled(true);
   actionIniciar->setEnabled(true);
   actionParar->setEnabled(false);
   filePath = ".";
	statusBar()->showMessage(tr("Pronto"));
	}

MainWindow::~MainWindow()
   {
   }

void MainWindow::statusMessage(QString strMessage)
   {
   statusBar()->showMessage(strMessage);
   }

void MainWindow::closeEvent(QCloseEvent *event)
	{
   if (deviceOpened)
      {
      voipManager->closeDevice();
      deviceOpened = false;
      }
   if (fileOpened)
      {
      fileOpened = false;
      }
   event->accept();
	}

void MainWindow::actOpenFile(void)
	{
	QFileDialog fileDialog;
	QString fileName;

	fileName = fileDialog.getOpenFileName(mdiArea, tr("Abrir Arquivo"), filePath, tr("Arquivo de captura (*.pcap)"));

	if (!fileName.isEmpty())
		{
      if (fileOpened)
         {
         fileOpened = false;
         }
      if (windowTrans != NULL)
         {
         windowTrans->clearTransactions();
         voipManager->clearTransactions();         
         }

		if (voipManager->openFile(fileName.toLatin1().data(), "udp") == 0)
         {
         timeAnalysis.start();
			filePath = fileName;
         if (windowTrans == NULL)
            {
            windowTrans = new WindowTransactions();
            subWindowTrans = mdiArea->addSubWindow(windowTrans);
            subWindowTrans->setWindowFlags((subWindowTrans->windowFlags()^Qt::WindowMaximizeButtonHint)
                                         | Qt::MSWindowsFixedSizeDialogHint
                                         | Qt::WindowMinimizeButtonHint
                                         | Qt::WindowCloseButtonHint
                                         | Qt::CustomizeWindowHint);
            subWindowTrans->show();

				connect(voipManager, SIGNAL(sendTransaction(int, char*,char*,char*,char*,char*,char*,char*)), windowTrans, SLOT(addPacket(int, char*,char*,char*,char*,char*,char*,char*)));
				connect(voipManager, SIGNAL(sendTransactionComplete(int, char*,char*,char*,char*,char*,char*,char*,char*)), windowTrans, SLOT(addPacketComplete(int, char*,char*,char*,char*,char*,char*,char*,char*)));
				connect(voipManager, SIGNAL(sendUpdateTransaction(int,char*)), windowTrans, SLOT(rxUpdateTransaction(int,char*)));
            connect(voipManager, SIGNAL(analysisFinishedSignal(int, int)), this, SLOT(analysisFinishedSignal(int, int)));
            connect(voipManager, SIGNAL(statusMessage(QString)), this, SLOT(statusMessage(QString)));
            connect(windowTrans, SIGNAL(closingTrans(bool)), this, SLOT(closingTransactions(bool)));
				connect(windowTrans, SIGNAL(transSelected(QList<int>)), this, SLOT(transSelected(QList<int>)));
            connect(windowTrans, SIGNAL(transSelectedLastPacket(QList<int>)), this, SLOT(transSelectedLastPacket(QList<int>)));
				connect(windowTrans, SIGNAL(transSelectedReport(QList<int>)), this, SLOT(transSelectedReport(QList<int>)));
				connect(windowTrans, SIGNAL(transSelectedPlay(int)), this, SLOT(transSelectedPlay(int)));            
				connect(windowTrans, SIGNAL(transFilter()), this, SLOT(transFilter()));

				actionEstatisticas->setEnabled(true);
				actionFiltro->setEnabled(true);

            connect (actionEstatisticas, SIGNAL(triggered()), this, SLOT (transStatistics()));
				connect (actionFiltro, SIGNAL(triggered()), this, SLOT(transFilter()));

				actionAbrir->setEnabled(false);
            actionAbrir_Sequencial->setEnabled(false);
            actionIniciar->setEnabled(false);
            actionParar->setEnabled(false);
            }
         fileOpened = true;
         }
		}
	}

void MainWindow::actOpenSequentialFiles(void)
	{
	QFileDialog fileDialog;
	QStringList fileNames;

	fileNames = fileDialog.getOpenFileNames(mdiArea, tr("Abrir Arquivo"), filePath, tr("Arquivo de captura (*.pcap)"));

	if (!fileNames.isEmpty())
		{
		if (fileOpened)
			{
			fileOpened = false;
			}
		if (windowTrans != NULL)
			{
			windowTrans->clearTransactions();
			voipManager->clearTransactions();   
			}

		if (voipManager->openFileSequential(fileNames, "udp") == 0)
			{
         timeAnalysis.start();
			filePath = fileNames.at(0);
			if (windowTrans == NULL)
				{
				windowTrans = new WindowTransactions();
				subWindowTrans = mdiArea->addSubWindow(windowTrans);
            subWindowTrans->setWindowFlags((subWindowTrans->windowFlags()^Qt::WindowMaximizeButtonHint)
                                         | Qt::MSWindowsFixedSizeDialogHint
                                         | Qt::WindowMinimizeButtonHint
                                         | Qt::WindowCloseButtonHint
                                         | Qt::CustomizeWindowHint);
				subWindowTrans->show();

				connect(voipManager, SIGNAL(sendTransaction(int, char*,char*,char*,char*,char*,char*,char*)), windowTrans, SLOT(addPacket(int, char*,char*,char*,char*,char*,char*,char*)));
				connect(voipManager, SIGNAL(sendTransactionComplete(int, char*,char*,char*,char*,char*,char*,char*,char*)), windowTrans, SLOT(addPacketComplete(int, char*,char*,char*,char*,char*,char*,char*,char*)));
				connect(voipManager, SIGNAL(sendUpdateTransaction(int,char*)), windowTrans, SLOT(rxUpdateTransaction(int,char*)));
            connect(voipManager, SIGNAL(analysisFinishedSignal(int, int)), this, SLOT(analysisFinishedSignal(int, int)));
            connect(voipManager, SIGNAL(statusMessage(QString)), this, SLOT(statusMessage(QString)));
				connect(windowTrans, SIGNAL(closingTrans(bool)), this, SLOT(closingTransactions(bool)));
				connect(windowTrans, SIGNAL(transSelected(QList<int>)), this, SLOT(transSelected(QList<int>)));
            connect(windowTrans, SIGNAL(transSelectedLastPacket(QList<int>)), this, SLOT(transSelectedLastPacket(QList<int>)));
				connect(windowTrans, SIGNAL(transSelectedReport(QList<int>)), this, SLOT(transSelectedReport(QList<int>)));
				connect(windowTrans, SIGNAL(transSelectedPlay(int)), this, SLOT(transSelectedPlay(int)));
				connect(windowTrans, SIGNAL(transFilter()), this, SLOT(transFilter()));

				actionEstatisticas->setEnabled(true);
				actionFiltro->setEnabled(true);

            connect (actionEstatisticas, SIGNAL(triggered()), this, SLOT (transStatistics()));
				connect (actionFiltro, SIGNAL(triggered()), this, SLOT(transFilter()));

				actionAbrir->setEnabled(false);
            actionAbrir_Sequencial->setEnabled(false);
				actionIniciar->setEnabled(false);
				actionParar->setEnabled(false);
				}
			fileOpened = true;
			}
		}
	}

void MainWindow::actAbout(void)
   {
   actionAbout->setEnabled(false);
   winAbout = new windowAbout();
   subWindowAbout = mdiArea->addSubWindow(winAbout);
   subWindowAbout->setWindowFlags((subWindowAbout->windowFlags()^Qt::WindowMaximizeButtonHint)
                                | Qt::MSWindowsFixedSizeDialogHint
                                | Qt::WindowMinimizeButtonHint
                                | Qt::WindowCloseButtonHint
                                | Qt::CustomizeWindowHint);
   subWindowAbout->show();

   connect (winAbout, SIGNAL(aboutClose(bool)), this, SLOT(closingAbout(bool)));
   }

void MainWindow::actManual(void)
   {
   //QDesktopServices::openUrl(QUrl("file:///D://pcs//Mestrado//projeto//produes//VoIPFix dissertao_versao_1_0.pdf", QUrl::TolerantMode));
   }

void MainWindow::actWebsite(void)
   {
   QDesktopServices::openUrl(QUrl("http://ccsl.ime.usp.br/voipfix"));
   }

void MainWindow::actSaveFile(void)
   {
	QFileDialog fileDialog;
	QString fileName;

	fileName = fileDialog.getSaveFileName(mdiArea, tr("Salvar Arquivo"), ".", tr("Arquivo de captura (*.pcap)"));

	if (!fileName.isEmpty())
		voipManager->saveFile(fileName.toLatin1().data());
   }

void MainWindow::actStartCapture(void)
	{
	QStringList *deviceList;

   if ((!deviceOpened) && (!fileOpened))
      {
      deviceList = voipManager->getDeviceList();

      actionIniciar->setEnabled(false);

      windowCap = new WindowCapture();
      subWindowCap = mdiArea->addSubWindow(windowCap);
      subWindowCap->setWindowFlags((subWindowCap->windowFlags()^Qt::WindowMaximizeButtonHint)
                                   | Qt::MSWindowsFixedSizeDialogHint
                                   | Qt::WindowMinimizeButtonHint
                                   | Qt::WindowCloseButtonHint
                                   | Qt::CustomizeWindowHint);
      subWindowCap->show();

      windowCap->addDevice(deviceList);
		connect(windowCap, SIGNAL(configCapture(int, char*)), this, SLOT(configureCapture(int, char*)));
      connect(windowCap, SIGNAL(configCancel(bool)), this, SLOT(configureCancel(bool)));
      }
	}

void MainWindow::actStopCapture(void)
   {
   if (deviceOpened)
      {
      voipManager->closeDevice();
      deviceOpened = false;
      actionAbrir->setEnabled(true);
      actionAbrir_Sequencial->setEnabled(true);
      actionIniciar->setEnabled(true);
      actionParar->setEnabled(false);
      }
   }

void MainWindow::actClose(void)
   {
   if (deviceOpened)
      {
      voipManager->closeDevice();
      deviceOpened = false;
      }
   if (fileOpened)
      {
      fileOpened = false;
      }
   close();
   }

void MainWindow::configureCapture(int device, char * filter)
	{
   if (voipManager->openDevice(device, 1, filter) == 0)
      {      
      disconnect(windowCap, SIGNAL(configCapture(int, char *)), this, SLOT(configureCapture(int, char *)));
      subWindowCap->close();
      delete subWindowCap;

      if (windowTrans != NULL)
         {
         windowTrans->clearTransactions();
         voipManager->clearTransactions();         
         }
      else
         {
         windowTrans = new WindowTransactions();
         QMdiSubWindow *subWindow = mdiArea->addSubWindow(windowTrans);
         subWindow->setWindowFlags((subWindow->windowFlags()^Qt::WindowMaximizeButtonHint)
                                      | Qt::MSWindowsFixedSizeDialogHint
                                      | Qt::WindowMinimizeButtonHint
                                      | Qt::WindowCloseButtonHint
                                      | Qt::CustomizeWindowHint);
         subWindow->show();

			connect(voipManager, SIGNAL(sendTransaction(int, char*,char*,char*,char*,char*,char*,char*)), windowTrans, SLOT(addPacket(int, char*,char*,char*,char*,char*,char*,char*)));
			connect(voipManager, SIGNAL(sendTransactionComplete(int, char*,char*,char*,char*,char*,char*,char*,char*)), windowTrans, SLOT(addPacketComplete(int, char*,char*,char*,char*,char*,char*,char*,char*)));
			connect(voipManager, SIGNAL(sendUpdateTransaction(int,char*)), windowTrans, SLOT(rxUpdateTransaction(int,char*)));
         connect(voipManager, SIGNAL(statusMessage(QString)), this, SLOT(statusMessage(QString)));
         connect(windowTrans, SIGNAL(closingTrans(bool)), this, SLOT(closingTransactions(bool)));
			connect(windowTrans, SIGNAL(transSelected(QList<int>)), this, SLOT(transSelected(QList<int>)));
         connect(windowTrans, SIGNAL(transSelectedLastPacket(QList<int>)), this, SLOT(transSelectedLastPacket(QList<int>)));
			connect(windowTrans, SIGNAL(transSelectedReport(QList<int>)), this, SLOT(transSelectedReport(QList<int>)));
			connect(windowTrans, SIGNAL(transSelectedPlay(int)), this, SLOT(transSelectedPlay(int)));
			connect(windowTrans, SIGNAL(transFilter()), this, SLOT(transFilter()));
			}

		actionEstatisticas->setEnabled(true);
		actionFiltro->setEnabled(true);

      connect (actionEstatisticas, SIGNAL(triggered()), this, SLOT (transStatistics()));
		connect (actionFiltro, SIGNAL(triggered()), this, SLOT(transFilter()));

		actionAbrir->setEnabled(false);
      actionAbrir_Sequencial->setEnabled(false);
      actionIniciar->setEnabled(false);
      actionParar->setEnabled(true);

      deviceOpened = true;
      }
	}

void MainWindow::configureCancel(bool closing)
   {
   actionIniciar->setEnabled(true);
   if (deviceOpened)
      {
      voipManager->closeDevice();
      deviceOpened = false;
      }
   disconnect(windowCap, SIGNAL(configCapture(int, char *)), this, SLOT(configureCapture(int, char *)));
   if (!closing)
      {
      subWindowCap->close();
      delete subWindowCap;
      }
   }

void MainWindow::closingTransactions(bool closing)
   {
   if (deviceOpened)
      {
      voipManager->closeDevice();
      deviceOpened = false;
      }
   if (fileOpened)
      {
      fileOpened = false;
      }

	disconnect(voipManager, SIGNAL(sendTransaction(int, char*,char*,char*,char*,char*,char*,char*)), windowTrans, SLOT(addPacket(int, char*,char*,char*,char*,char*,char*,char*)));
	disconnect(voipManager, SIGNAL(sendTransactionComplete(int, char*,char*,char*,char*,char*,char*,char*,char*)), windowTrans, SLOT(addPacketComplete(int, char*,char*,char*,char*,char*,char*,char*,char*)));
	disconnect(voipManager, SIGNAL(sendUpdateTransaction(int,char*)), windowTrans, SLOT(rxUpdateTransaction(int,char*)));
   disconnect(voipManager, SIGNAL(analysisFinishedSignal(int, int)), this, SLOT(analysisFinishedSignal(int, int)));
   disconnect(voipManager, SIGNAL(statusMessage(QString)), this, SLOT(statusMessage(QString)));
	disconnect(windowTrans, SIGNAL(closingTrans(bool)), this, SLOT(closingTransactions(bool)));
	disconnect(windowTrans, SIGNAL(transSelected(QList<int>)), this, SLOT(transSelected(QList<int>)));
   disconnect(windowTrans, SIGNAL(transSelectedLastPacket(QList<int>)), this, SLOT(transSelectedLastPacket(QList<int>)));
	disconnect(windowTrans, SIGNAL(transSelectedReport(QList<int>)), this, SLOT(transSelectedReport(QList<int>)));
	disconnect(windowTrans, SIGNAL(transSelectedPlay(int)), this, SLOT(transSelectedPlay(int)));
	disconnect(windowTrans, SIGNAL(transFilter()), this, SLOT(transFilter()));

   windowTrans->clearTransactions();
   delete windowTrans;
   windowTrans = NULL;
   voipManager->clearTransactions();   

	actionEstatisticas->setEnabled(false);
	actionFiltro->setEnabled(false);

   disconnect (actionEstatisticas, SIGNAL(triggered()), this, SLOT (transStatistics()));
	disconnect (actionFiltro, SIGNAL(triggered()), this, SLOT(transFilter()));

	actionAbrir->setEnabled(true);
   actionAbrir_Sequencial->setEnabled(true);
   actionIniciar->setEnabled(true);
   actionParar->setEnabled(false);
}

void MainWindow::transSelected (QList<int> transSel)
	{
	QList <infoPacketTrans_t> infoPacketTrans;
	infoPacketTrans = voipManager->setTransSelected(transSel);

	if (winGraph == NULL)
		{
		winGraph = new WindowGraph();
		subWindowGraph = mdiArea->addSubWindow(winGraph);      
      subWindowGraph->setGeometry(winGraph->setInfoGraph(infoPacketTrans));
		subWindowGraph->setMinimumSize(subWindowGraph->size());
      subWindowGraph->show();

		connect(winGraph, SIGNAL(closingGraph(bool)), this, SLOT(closingGraph(bool)));
		connect(winGraph, SIGNAL(packetClicked(int)), this, SLOT(packetClicked(int)));
		}
	}

void MainWindow::transSelectedLastPacket (QList<int> transSel)
   {
   QList <infoPacketTrans_t> infoPacketTrans;
   infoPacketTrans = voipManager->setTransSelectedLastPacket(transSel);

	if (winPackets == NULL)
      {
		winPackets = new windowPackets();
		subWindowPackets = mdiArea->addSubWindow(winPackets);
		winPackets->setInfoPackets(infoPacketTrans, voipManager->getListPackets());
      subWindowPackets->setWindowFlags((subWindowPackets->windowFlags()^Qt::WindowMaximizeButtonHint)
                                   | Qt::MSWindowsFixedSizeDialogHint
                                   | Qt::WindowMinimizeButtonHint
                                   | Qt::WindowCloseButtonHint
                                   | Qt::CustomizeWindowHint);
		subWindowPackets->show();

		connect(winPackets, SIGNAL(closingPackest(bool)), this, SLOT(closingPackets(bool)));
      }
   }

void MainWindow::transSelectedReport(QList<int> transSel)
	{
	QList <infoReportTrans_t> infoReport;
	infoReport = voipManager->setTransSelectedReport(transSel);

	if (winReport == NULL)
		{
		winReport = new WindowReport();
		subWindowReport = mdiArea->addSubWindow(winReport);
		winReport->setInfoPackets(infoReport, voipManager->getListPackets());
      subWindowReport->setWindowFlags((subWindowReport->windowFlags()^Qt::WindowMaximizeButtonHint)
                                   | Qt::MSWindowsFixedSizeDialogHint
                                   | Qt::WindowMinimizeButtonHint
                                   | Qt::WindowCloseButtonHint
                                   | Qt::CustomizeWindowHint);
		subWindowReport->show();

		connect(winReport, SIGNAL(closingReport(bool)), this, SLOT(closingReport(bool)));
		}
	}

void MainWindow::transSelectedPlay(int transSel)
	{
	stream_RTP_t streamRTP;
	streamRTP = voipManager->setTransSelectedPlay(transSel);

	if (winPlay == NULL)
		{
		winPlay = new WindowPlay();
		subWindowPlay = mdiArea->addSubWindow(winPlay);
		winPlay->setInfoPlay(streamRTP);
      subWindowPlay->setWindowFlags((subWindowPlay->windowFlags()^Qt::WindowMaximizeButtonHint)
                                   | Qt::MSWindowsFixedSizeDialogHint
                                   | Qt::WindowMinimizeButtonHint
                                   | Qt::WindowCloseButtonHint
                                   | Qt::CustomizeWindowHint);
		subWindowPlay->show();
		connect(winPlay, SIGNAL(closingPlay(bool)), this, SLOT(closingPlay(bool)));
		}
	}

void MainWindow::transFilter(void)
	{
	if (winFilter == NULL)
		{
		winFilter = new WindowFilter();
		subWindowFilter = mdiArea->addSubWindow(winFilter);
      subWindowFilter->setWindowFlags((subWindowFilter->windowFlags()^Qt::WindowMaximizeButtonHint)
                                   | Qt::MSWindowsFixedSizeDialogHint
                                   | Qt::WindowMinimizeButtonHint
                                   | Qt::WindowCloseButtonHint
                                   | Qt::CustomizeWindowHint);
		subWindowFilter->show();
		connect (winFilter, SIGNAL(closingFilter(bool)), this, SLOT(closingFilter(bool)));
		connect (winFilter, SIGNAL(paramFilter(transFilter_t)), this, SLOT(parameterFilter(transFilter_t)));
		}
	}

void MainWindow::transStatistics(void)
	{
   if (winStatistics == NULL)
		{
      winStatistics = new WindowStatistics();
      winStatistics->setInfoTrans(voipManager->getListTrans());
      subWindowStatistics = mdiArea->addSubWindow(winStatistics);
      subWindowStatistics->setWindowFlags((subWindowStatistics->windowFlags()^Qt::WindowMaximizeButtonHint)
                                   | Qt::MSWindowsFixedSizeDialogHint
                                   | Qt::WindowMinimizeButtonHint
                                   | Qt::WindowCloseButtonHint
                                   | Qt::CustomizeWindowHint);
      subWindowStatistics->show();
      connect(winStatistics, SIGNAL(closingStatistics(bool)), this, SLOT(closingStatistics(bool)));
		}
	}

void MainWindow::closingGraph (bool closing)
	{
	disconnect(winGraph, SIGNAL(closingGraph(bool)), this, SLOT(closingGraph(bool)));

	delete winGraph;
	winGraph = NULL;
	}

void MainWindow::closingPackets(bool closing)
	{
	disconnect(winPackets, SIGNAL(closingPackest(bool)), this, SLOT(closingPackets(bool)));

	delete winPackets;
	winPackets = NULL;
	}

void MainWindow::closingReport(bool closing)
	{
	disconnect(winReport, SIGNAL(closingReport(bool)), this, SLOT(closingReport(bool)));

	delete winReport;
	winReport = NULL;
	}

void MainWindow::closingPlay(bool closing)
	{
	disconnect(winPlay, SIGNAL(closingPlay(bool)), this, SLOT(closingPlay(bool)));

	delete winPlay;
	winPlay = NULL;
	}

void MainWindow::closingFilter (bool closing)
	{
	disconnect(winFilter, SIGNAL(closingFilter(bool)), this, SLOT(closingFilter(bool)));

	delete winFilter;
	winFilter = NULL;
	subWindowFilter->close();
	}

void MainWindow::closingStatistics (bool closing)
	{
   disconnect(winStatistics, SIGNAL(closingStatistics(bool)), this, SLOT(closingStatistics(bool)));

   delete winStatistics;
   winStatistics = NULL;
	}

void MainWindow::closingAbout (bool closing)
   {
   actionAbout->setEnabled(true);
   disconnect(winAbout, SIGNAL(aboutClose(bool)), this, SLOT(closingAbout(bool)));

   delete winAbout;
   winAbout = NULL;
   }

void MainWindow::packetClicked(int numPacket)
	{
	if (winPackets != NULL)
		{
		winPackets->setPacketClicked(numPacket);
		}
	}

void MainWindow::analysisFinishedSignal(int numSIP, int numRTP)
   {
   double valTime;
   QString strAux;
   valTime = (double)((double)timeAnalysis.elapsed()/1000);

	strAux = QString(tr("Tempo para abrir e analisar arquivo: %1 [s] - Nmero de pacotes SIP=[%2] - Nmero de pacotes RTP=[%3]").arg(valTime).arg(numSIP).arg(numRTP));
   statusBar()->showMessage(strAux);
   }

void MainWindow::parameterFilter (transFilter_t tFilter)
	{
	windowTrans->clearTransactions();
	voipManager->paramFilter(tFilter);
	}
