#include <osipparser2/osip_message.h>
#include <osipparser2/sdp_message.h>
#include <osipparser2/osip_parser.h>

#include <stdlib.h>

int osip_call_id_to_str (const osip_message * sip, char **dest)
	{
	if (dest == NULL)
		return (-1);

	if ((sip == NULL) || (sip->call_id->number == NULL))
		return -1;

	if (sip->call_id->host == NULL)
		{
		*dest = (char *) malloc (strlen (sip->call_id->number) + 1);
		if (*dest == NULL)
			return -1;
		sprintf (*dest, "%s", sip->call_id->number);
		}
	else
		{
		*dest = (char *) malloc (strlen (sip->call_id->number) + strlen (sip->call_id->host) + 2);
		if (*dest == NULL)
			return -1;
		sprintf (*dest, "%s@%s", sip->call_id->number, sip->call_id->host);
		}
	return 0;
	}

char * osip_message_get_content_type_subtype(osip_message_t * sip)
	{
	if (sip->content_type != NULL)
		{
		return (sip->content_type->subtype);
		}
	else
		{
		return NULL;
		}
	}

char * strnstr(const char *s, const char *find, size_t slen)
	{
	char c, sc;
	size_t len;

	if ((c = *find++) != '\0') {
		len = strlen(find);
		do {
			do {
				if (slen-- < 1 || (sc = *s++) == '\0')
					return (NULL);
			} while (sc != c);
			if (len > slen)
				return (NULL);
		} while (strncmp(s, find, len) != 0);
		s--;
	}
	return ((char *)s);
	}

int get_first_line(char *payload, char **first_line)
	{
	char * pos_end;
	int Ret, tam_first_line;
	Ret = -1;
	if (payload != NULL)
		{
		pos_end = strnstr(payload, "\r\n", 120);
		if (pos_end != NULL)
			{
			tam_first_line = pos_end - payload;
			*first_line = (char*)malloc (tam_first_line + 2);
			memcpy (*first_line, payload, tam_first_line);
			Ret = tam_first_line;
			}
		}
	return Ret;
	}

char * get_payload_type_str(unsigned char payload)
	{
	switch (payload)
		{
		case 0: return ("g711u");
		case 3: return ("GSM");
		case 4: return ("G723");
		case 5: return ("DVI4");
		case 6: return ("DVI4");
		case 7: return ("LPC");
		case 8: return ("g711a");
		case 9: return ("G722");
		case 10: return ("L16");
		case 11: return ("L16");
		case 12: return ("QCELP");
		case 13: return ("CN");
		case 14: return ("MPA");
		case 15: return ("G728");
		case 16: return ("DVI4");
		case 17: return ("DVI4");
		case 18: return ("G729");
//		case dyn G726-40
//		case dyn G726-32
//		case dyn G726-24
//		case dyn G726-16
//		case dyn G729D
//		case dyn G729E
//		case dyn GSM-EFR
//		case dyn L8
//		case dyn RED
//		case dyn VDVI
		case 25: return ("CelB");
		case 26: return ("JPEG");
		case 31: return ("H261");
		case 32: return ("MPV");
		case 33: return ("MP2T");
		case 34: return ("H263");
		case 97: return ("iLBC");
		case 98: return ("iLBC");
		case 101: return ("tel-event");
		default: return (NULL);
		}
	}


