#ifndef STRUCTS_H
#define STRUCTS_H

//Structures

// Ethernet header
struct sniff_ethernet
   {
   u_char ether_dhost[ETHER_ADDR_LEN]; /* Destination host address */
   u_char ether_shost[ETHER_ADDR_LEN]; /* Source host address */
   u_short ether_type; /* IP? ARP? RARP? etc */
   };

union IPADDRESS_t
   {
	unsigned int ipdw;
	unsigned char ip[4];
	};

/* IP header */
struct sniff_ip
   {
   u_char ip_vhl;		/* version << 4 | header length >> 2 */
   u_char ip_tos;		/* type of service */
   u_short ip_len;		/* total length */
   u_short ip_id;		/* identification */
   u_short ip_off;		/* fragment offset field */
   #define IP_RF 0x8000		/* reserved fragment flag */
   #define IP_DF 0x4000		/* dont fragment flag */
   #define IP_MF 0x2000		/* more fragments flag */
   #define IP_OFFMASK 0x1fff	/* mask for fragmenting bits */
   u_char ip_ttl;		/* time to live */
   u_char ip_p;		/* protocol */
   u_short ip_sum;		/* checksum */
   //struct in_addr ip_src,ip_dst; /* source and dest address */
   IPADDRESS_t ip_src,ip_dst;
   };
   #define IP_HL(ip)		(((ip)->ip_vhl) & 0x0f)
   #define IP_V(ip)		(((ip)->ip_vhl) >> 4)


// UDP header
struct sniff_udp
   {
   u_short udp_sport;	// source port
   u_short udp_dport;	// destination port
   u_short udp_len;		/* total length */
   u_short udp_sum;		/* checksum */
   };

typedef struct
	{
	IPADDRESS_t ip_src;
	IPADDRESS_t ip_dst;
	u_short udp_sport;	// source port
	u_short udp_dport;	// destination port
	u_short udp_len;		/* total length */
	}infoUDP_t;

typedef struct
   {
   unsigned char version;    /* protocol version */
   unsigned char p;          /* padding flag */
   unsigned char x;          /* header extension flag */
   unsigned char cc;         /* CSRC count */
   unsigned char m;          /* marker bit */
   unsigned char pt;         /* payload type */
   unsigned short seq;       /* sequence number */
	unsigned int ts;          /* timestamp */
	unsigned int ssrc;        /* synchronization source */
   }infoRTP_t;

typedef struct
	{
   struct pcap_pkthdr *header;
	u_char *pkt_data;
	osip_message_t * sip_message;
	sdp_message_t * sdp_message;
	infoUDP_t * infoUDP;
   infoRTP_t * infoRTP;
	int numPacote;
   }packet_t;

typedef struct
   {
   char * call_id;
	int tipoTrans;
   int statusCode;
	int numPacketReq;
	QList <int> posPackets;
	IPADDRESS_t ipMediaSrc;
	IPADDRESS_t ipMediaDst;
	int portMediaSrc;
	int portMediaDst;
	bool mediaSrcOK;
	bool mediaDstOK;
	unsigned int ssrcSrc;
	unsigned int ssrcDst;
	QReadWriteLock *lock;
   }transaction_t;

typedef struct
	{
	int numPacket;
	int numTrans;
	timeval time;
	IPADDRESS_t srcIP;
	u_short srcPort;
	IPADDRESS_t dstIP;
	u_short dstPort;
	QString info;
	QString coment;
   }infoPacketTrans_t;


typedef struct
	{
	int numTrans;
	QString tStart;
	QString tStop;
	QString Tipo;
	QString IPFrom;
	QString userFrom;
	QString IPTo;
	QString userTo;
	QString state;
	QString duration;

	QString Codec;

	QString meanJitterOrg;
	QString maxJitterOrg;
	QString packetLostPercOrg;
	QString sequenceErrorOrg;

	QString meanJitterRec;
	QString maxJitterRec;
	QString packetLostPercRec;
	QString sequenceErrorRec;

	//report de erros
	int erroRTP;
	QString erroRTPText;

	int erroSIP;
	QString erroSIPText;
	}infoReportTrans_t;

typedef struct
   {
   int numTrans;
   QList <unsigned int> lstSSRC;
   }contSSRC_t;

typedef struct
	{
	int numOccurrence;
	QString user;
	}estUser_t;

typedef struct
	{
	int numOccurrence;
	IPADDRESS_t ip;
	}estIP_t;

typedef struct
	{
	unsigned short max_seq;			// highest seq. number seen
	unsigned int cycles;				// shifted count of seq. number cycles
	unsigned int base_seq;			// base seq number
	unsigned int bad_seq;			// last 'bad' seq number + 1
	unsigned int probation;			// sequ. packets till source is valid
	unsigned int received;			// packets received
	unsigned int expected_prior;	// packet expected at last interval
	unsigned int received_prior;	// packet received at last interval
	unsigned int sequenceError;	//sequence error counts
	double transit;			// relative trans time for prev pkt
	double jitter;				// estimated jitter
	double meanJitter;		// calculation of mean jitter
	double maxJitter;			// calculation of max jitter
	unsigned int timeStamp;
	bool absence;				//signals the long absence of RTP
									//number of packets higher than defined in RTP_MAX_INTERVALO
	} sourceRTP_t;

typedef struct
	{
	IPADDRESS_t ip;
	int state;
	int status;
	QList <int> codecs;
	int registerTime;
	int CSeq;
	} transState_t;

typedef struct
	{
	QList <int> originator;
	int codecOriginator;
	QList <int> receiver;
	int codecReceiver;
	}stream_RTP_t;

typedef struct
	{
	int numTrans;
	struct timeval tStart;
	struct timeval tStop;
	QString sipMethod;
	IPADDRESS_t ipFrom;
	IPADDRESS_t ipTo;
	QString userFrom;
	QString userTo;
	int state;
	}transList_t;

typedef struct
	{
	//seach interval
	QTime tStart;	//start time search
	QTime tStop;	//end time search

	bool Calls;
	bool Register;
	bool All;

	//host address of search
	IPADDRESS_t ipFrom;	//source address search
	bool isIpFrom;			//true to search
	bool ipFromEqual;    //true to search with equal value, false for different

	IPADDRESS_t ipTo;		//destination address search
	bool isIpTo;			//true to search
	bool ipToEqual;      //true to search with equal value, false for different
	bool ipFromAndIpTo;	//true to search source and destination IP with these values

	QString userFrom;		//source user search
	bool isUserFrom;		//true to search
	bool userFromEqual;  //true to search with equal value, false for different

	QString userTo;		//destination user search
	bool isUserTo;			//true to search
	bool usertoEqual;    //true to search with equal value, false for different
	bool userFromAndUserTo;	//true to search source and destination user with these values

	//status of completion of transactions to be searched
	bool statusAll;
	bool statusFinished;
	bool statusCompleted;
	bool statusBusy;
	bool statusCanceled;
	bool statusRefused;
	bool statusNotAuthorized;
	bool statusUnsupportedMedia;
	}transFilter_t;
#endif // STRUCTS_H
