#include "voipfixmanager.h"
#include <QStringList>

#ifdef Q_OS_UNIX
#include <arpa/inet.h>
#include <strings.h>
#endif

#include <osip_utils.h>

void sip_parse(packet_t &pacote_parse)
   {
   sniff_ethernet *header_ethernet;
   sniff_ip *header_ip;
   sniff_udp *header_udp;
   u_char *payload;
   int size_ip;

   header_ethernet = (struct sniff_ethernet*) pacote_parse.pkt_data;
   if (ntohs(header_ethernet->ether_type) == 0x0800)
      {
      //IP Start
      header_ip = (struct sniff_ip*)(pacote_parse.pkt_data + SIZE_ETHERNET);
      size_ip = IP_HL(header_ip)*4;
      if (size_ip >= 20)
         {
         if (header_ip->ip_p == IPPROTO_UDP)
            {
            header_udp = (struct sniff_udp*)(pacote_parse.pkt_data + SIZE_ETHERNET + size_ip);

				pacote_parse.infoUDP = (infoUDP_t*)malloc(sizeof(infoUDP_t));
				pacote_parse.infoUDP->ip_src.ipdw = header_ip->ip_src.ipdw;
				pacote_parse.infoUDP->ip_dst.ipdw = header_ip->ip_dst.ipdw;
            pacote_parse.infoUDP->udp_sport = ntohs(header_udp->udp_sport);
            pacote_parse.infoUDP->udp_dport = ntohs(header_udp->udp_dport);
            pacote_parse.infoUDP->udp_len = ntohs(header_udp->udp_len);

            payload = (u_char*)(pacote_parse.pkt_data + SIZE_ETHERNET + size_ip + 8);
            if ((strncmp((const char*)payload, (const char*)"SIP/2.0", 8) == 0) || (strnstr((const char*)payload, (const char*)"SIP/2.0", 120) != NULL))
               {
               osip_message_init (&pacote_parse.sip_message);               
               if (osip_message_parse(pacote_parse.sip_message, (char*)payload, ntohs(header_udp->udp_len) - 8) == OSIP_SUCCESS)
                  {
						char * content_subtype;

						if ((content_subtype = osip_message_get_content_type_subtype(pacote_parse.sip_message)) != NULL)
                     {
#ifdef Q_OS_WIN
                     if (stricmp(content_subtype, "sdp") == 0)
#endif
#ifdef Q_OS_UNIX
                     if (strcasecmp(content_subtype, "sdp") == 0)
#endif
                        {
                        osip_body_t * body;
                        sdp_message_init(&pacote_parse.sdp_message);
                        body = (osip_body_t*)pacote_parse.sip_message->bodies.node->element;
                        if (sdp_message_parse(pacote_parse.sdp_message, body->body) != OSIP_SUCCESS)
                           {
                           sdp_message_free(pacote_parse.sdp_message);
                           pacote_parse.sdp_message = NULL;
                           }
                        }
                     }
                  }
               else
                  {
                  osip_message_free(pacote_parse.sip_message);
                  pacote_parse.sip_message = NULL;
                  }
               }
				else
               {	//isn't SIP, but UDP... Check if is RTP
               if (pacote_parse.infoUDP->udp_len >= 12)
                  {  //check if the payload has at least the size of a header RTP
                  pacote_parse.infoRTP = (infoRTP_t*)malloc(sizeof(infoRTP_t));
                  pacote_parse.infoRTP->version = (payload[0] & 0xC0) >> 6;
						pacote_parse.infoRTP->pt = (payload[1] & 0x7F);
						if ((pacote_parse.infoRTP->version == 0x02) && (get_payload_type_str(pacote_parse.infoRTP->pt) != NULL))
							{
							pacote_parse.infoRTP->p = (payload[0] & 0x20) >> 5;
							pacote_parse.infoRTP->x = (payload[0] & 0x10) >> 4;
							pacote_parse.infoRTP->cc = (payload[0] & 0x0F);
							pacote_parse.infoRTP->m = (payload[1] & 0x80) >> 7;
							pacote_parse.infoRTP->seq = (unsigned short) ((payload[2] << 8) + payload[3]);
							pacote_parse.infoRTP->ts = (unsigned int) ((payload[4] << 24) + (payload[5] << 16) + (payload[6] << 8) + payload[7]);
							pacote_parse.infoRTP->ssrc = (unsigned int) ((payload[8] << 24) + (payload[9] << 16) + (payload[10] << 8) + payload[11]);
							}
						else
							{
							free (pacote_parse.infoRTP);
							pacote_parse.infoRTP = NULL;
							}
                  }
					}
            }
         }
      }
   }

VoIPFixManager::VoIPFixManager()
	{

	}

VoIPFixManager::~VoIPFixManager()
	{
	delete voipCap;
	}

int VoIPFixManager::startManager()
   {
	parser_init();
	voipCap = new VoIPFixCap();
	voipCap->startCap();

	connect (voipCap, SIGNAL(winCapPacket(pcap_pkthdr*,const u_char*)), this, SLOT(rxPacket(pcap_pkthdr*,const u_char*)));
   connect (voipCap, SIGNAL(winCapEOF()), this, SLOT(rxEOF()));

	voipTransactions = new VoIPFixTransactions();
	voipTransactions->startTransactions();

	connect (voipTransactions, SIGNAL(findTransaction(int, char*,char*, char*, char*,char*,char*,char*)), this, SIGNAL(sendTransaction(int, char*,char*,char*,char*,char*,char*,char*)));
	connect (voipTransactions, SIGNAL(findTransactionComplete(int, char*,char*, char*, char*,char*,char*,char*,char*)), this, SIGNAL(sendTransactionComplete(int, char*,char*,char*,char*,char*,char*,char*,char*)));
	connect (voipTransactions, SIGNAL(updateTransaction(int,char*)), this, SIGNAL(sendUpdateTransaction(int,char*)));
   connect (voipTransactions, SIGNAL(analysisFinishedSignal(int, int)), this, SIGNAL(analysisFinishedSignal(int, int)));
   connect (voipTransactions, SIGNAL(statusMessage(QString)), this, SIGNAL(statusMessage(QString)));
	connect (&futureWatcher, SIGNAL(finished()), this, SLOT(parserFinished()));
	return 0;
   }

QStringList * VoIPFixManager::getDeviceList()
	{
	return voipCap->getDeviceList();
	}

int VoIPFixManager::openDevice (int deviceNumber, int inPromiscuousMode, char * filter)
	{
   int res;
	if ((res = voipCap->openDevice(deviceNumber, inPromiscuousMode, filter)) == 0)
      {
      openMode = OPEN_MODE_DEVICE;
      }
   return res;
	}

void VoIPFixManager::closeDevice (void)
	{
	voipCap->closeDevice();
	}

int VoIPFixManager::openFile (char * fileName, char * filter)
	{
	int res;
	if ((res = voipCap->openFile(fileName, filter)) == 0)
		{
      openMode = OPEN_MODE_FILE;
		}
	return res;
	}

int VoIPFixManager::openFileSequential (QStringList fileNames, char * filter)
	{
	int res;
   if ((res = voipCap->openFileSequential(fileNames, filter)) == 0)
		{
		openMode = OPEN_MODE_FILE;
		}
	return res;
	}

void VoIPFixManager::closeFile(void)
   {
   voipCap->closeFile();   
   }

int VoIPFixManager::saveFile (char * fileName)
   {
   int res = -1;
   int tamLista;
   int i;

   if (voipCap->saveFile(fileName) == 0)
      {
      tamLista = listaPacotes.size();
      for (i = 0; i < tamLista; i++)
         {
         voipCap->packetDumpFile(listaPacotes.value(i).header, listaPacotes.value(i).pkt_data);
         }
      voipCap->closeDumpFile();
      res = 0;
      }
   return res;
   }

void VoIPFixManager::rxPacket(pcap_pkthdr *header, const u_char *pkt_data)
	{
   if ((header != NULL) && (pkt_data != NULL))
      {
      pacote_aux.header = header;
      pacote_aux.pkt_data = (u_char*)pkt_data;

      pacote_aux.sip_message = NULL;
      pacote_aux.sdp_message = NULL;
		pacote_aux.infoUDP = NULL;
      pacote_aux.infoRTP = NULL;

      switch (openMode)
         {
         case OPEN_MODE_FILE:
            {            
				pacote_aux.numPacote = listaPacotes.size();
            listaPacotes.append(pacote_aux);            
            break;
            }
         case OPEN_MODE_DEVICE:
            {
            sip_parse(pacote_aux);
				pacote_aux.numPacote = listaPacotes.size();
            listaPacotes.append(pacote_aux);
            voipTransactions->rxPacket(pacote_aux);
            break;
            }
         case OPEN_MODE_NONE:
            {

            break;
            }
         default:
            {

            }
         }        
      }
	}

void VoIPFixManager::rxEOF(void)
   {   
	QProgressDialog dialog;
	dialog.setLabelText(QString(tr("Analisando arquivo(s) usando %1 thread(s)...")).arg(QThread::idealThreadCount()));
	dialog.setModal(true);

	connect(&futureWatcher, SIGNAL(finished()), &dialog, SLOT(reset()));
	connect(&dialog, SIGNAL(canceled()), &futureWatcher, SLOT(cancel()));
	connect(&futureWatcher, SIGNAL(progressRangeChanged(int, int)), &dialog, SLOT(setRange(int, int)));
	connect(&futureWatcher, SIGNAL(progressValueChanged(int)), &dialog, SLOT(setValue(int)));

	futureWatcher.setFuture(QtConcurrent::map(listaPacotes, sip_parse));

	dialog.exec();
	qApp->processEvents();
   }

void VoIPFixManager::parserFinished(void)
   {
   voipTransactions->rxPacketsList(listaPacotes);
   }

void VoIPFixManager::clearTransactions(void)
   {
   int i;
   int tamLista;

   tamLista = listaPacotes.size();

   for (i = 0; i < tamLista; i++)
      {
      if (listaPacotes.operator [](i).header != NULL)
         {
         free (listaPacotes.operator [](i).header);
         listaPacotes.operator [](i).header = NULL;
         }

      if (openMode == OPEN_MODE_FILE)
         {
         if (listaPacotes.operator [](i).pkt_data != NULL)
            {
            free (listaPacotes.operator [](i).pkt_data);
            listaPacotes.operator [](i).pkt_data = NULL;
            }
         }

      if (listaPacotes.operator [](i).sip_message != NULL)
         {
         osip_message_free (listaPacotes.operator [](i).sip_message);
         listaPacotes.operator [](i).sip_message = NULL;
         }

      if (listaPacotes.operator [](i).sdp_message != NULL)
         {
         sdp_message_free(listaPacotes.operator [](i).sdp_message);
         listaPacotes.operator [](i).sdp_message = NULL;
         }

      if (listaPacotes.operator [](i).infoUDP != NULL)
         {
         free (listaPacotes.operator [](i).infoUDP);
         listaPacotes.operator [](i).infoUDP = NULL;
         }

      if (listaPacotes.operator [](i).infoRTP != NULL)
         {
         free (listaPacotes.operator [](i).infoRTP);
         listaPacotes.operator [](i).infoRTP = NULL;
         }

      }
   listaPacotes.clear();
   voipTransactions->clearListTransactions();
   }

QList <infoPacketTrans_t> VoIPFixManager::setTransSelected (QList<int> transSel)
	{
	return (voipTransactions->setTransSelected(listaPacotes, transSel));
	}

QList <infoPacketTrans_t> VoIPFixManager::setTransSelectedLastPacket (QList<int> transSel)
   {
   return (voipTransactions->setTransSelectedLastPacket(listaPacotes, transSel));
   }

QList <infoReportTrans_t> VoIPFixManager::setTransSelectedReport (QList<int> transSel)
	{
	return (voipTransactions->setTransSelectedReport(listaPacotes, transSel));
	}

stream_RTP_t VoIPFixManager::setTransSelectedPlay (int transSel)
	{
	return (voipTransactions->setTransSelectedPlay(listaPacotes, transSel));
	}

QList <packet_t> VoIPFixManager::getListPackets (void)
	{
	return (listaPacotes);
	}

QList <transList_t> VoIPFixManager::getListTrans(void)
	{
	return (voipTransactions->getListTrans());
	}

void VoIPFixManager::paramFilter (transFilter_t tFilter)
	{
	voipTransactions->setParamFilter(tFilter);
	}
