#ifndef VOIPFIXMANAGER_H
#define VOIPFIXMANAGER_H

#include <QtGui>
#include <voipfixcap.h>
#include <voipfixtransactions.h>
#include <pcap.h>
#include <osipparser2/osip_message.h>
#include <osipparser2/sdp_message.h>
#include <osipparser2/osip_parser.h>

#include <defines.h>
#include <structs.h>

#define OPEN_MODE_NONE     0
#define OPEN_MODE_FILE     1
#define OPEN_MODE_DEVICE   2

class QStringList;

class VoIPFixManager : public QObject
	{
	Q_OBJECT
	public:
		VoIPFixManager();
		~VoIPFixManager();
      int startManager();
		QStringList * getDeviceList();
		int openDevice (int deviceNumber, int inPromiscuousMode, char * filter);
		void closeDevice (void);
		int openFile (char * fileName, char * filter);
      int openFileSequential (QStringList fileNames, char * filter);
		void closeFile (void);
      int saveFile (char * fileName);
      void clearTransactions(void);
		QList <infoPacketTrans_t> setTransSelected (QList<int> transSel);
      QList <infoPacketTrans_t> setTransSelectedLastPacket (QList<int> transSel);
		QList <infoReportTrans_t> setTransSelectedReport (QList<int> transSel);
		stream_RTP_t setTransSelectedPlay (int transSel);
		QList<packet_t> getListPackets (void);
		QList <transList_t> getListTrans(void);
		void paramFilter (transFilter_t tFilter);

   public slots:
      void rxPacket(pcap_pkthdr *header, const u_char *pkt_data);
      void rxEOF(void);
      void parserFinished (void);      

	signals:
		void sendTransaction(int numTrans, char * tStart, char * tStop, char * Tipo, char * IPFrom, char * userFrom, char * IPTo, char * userTo);
		void sendTransactionComplete(int numTrans, char * tStart, char * tStop, char * Tipo, char * IPFrom, char * userFrom, char * IPTo, char * userTo, char * State);
		void sendUpdateTransaction (int numTransaction, char * newState);
      void analysisFinishedSignal(int numSIP, int numRTP);
      void statusMessage (QString str);

	private:
		VoIPFixCap *voipCap;
		VoIPFixTransactions *voipTransactions;
      int openMode;
		QList<packet_t> listaPacotes;
      packet_t pacote_aux;
      struct pcap_pkthdr *header_aux;
      const u_char *pkt_data_aux;
		QFutureWatcher<void> futureWatcher;

	};

#endif // VOIPFIXMANAGER_H
