#ifndef VOIPFIXTRANSACTIONS_H
#define VOIPFIXTRANSACTIONS_H

#include <QtGui>
#include <pcap.h>
#include <osipparser2/osip_message.h>
#include <osipparser2/sdp_message.h>
#include <osipparser2/osip_parser.h>
#include <defines.h>
#include <structs.h>

#define RTP_SEQ_MOD		(1<<16)
#define MAX_DROPOUT		3000
#define MAX_MISORDER		100
#define MIN_SEQUENTIAL	2

class VoIPFixTransactions : public QObject
	{
	Q_OBJECT
	public:
		VoIPFixTransactions();
		~VoIPFixTransactions();
      int startTransactions();
      int rxPacketsList (QList<packet_t> PacketsList);
      void clearListTransactions(void);
		void quickSortInfoPackets(QList <infoPacketTrans_t> *infoP, int p, int r);
		int partitionInfoPackets(QList <infoPacketTrans_t> *infoP, int p, int r);
		void quickSortPackets(QList <int> *lstPackets, int p, int r);
		int partitionPackets(QList <int> *lstPackets, int p, int r);
      int rxPacket (packet_t packet);
      QList <infoPacketTrans_t> setTransSelected (QList<packet_t> packetsList, QList<int> transSel);
      QList <infoPacketTrans_t> setTransSelectedLastPacket (QList<packet_t> packetsList, QList<int> transSel);
      QList <infoReportTrans_t> setTransSelectedReport (QList<packet_t> packetsList, QList<int> transSel);
      stream_RTP_t setTransSelectedPlay (QList<packet_t> packetsList, int transSel);
      QList <int> getListTransPackets(int numTrans);
		QList <transList_t> getListTrans(void);
		void setParamFilter (transFilter_t tFilter);      

	private slots:
      void analysisFinished(QList<packet_t>);
      void analysisFinished2(void);

	signals:
      void findTransaction(int numTrans, char * tStart, char * tStop, char * Type, char * IPFrom, char * userFrom, char * IPTo, char * userTo);
      void findTransactionComplete(int numTrans, char * tStart, char * tStop, char * Type, char * IPFrom, char * userFrom, char * IPTo, char * userTo, char * State);
		void updateTransaction (int numTransaction, char * newState);
      void analysisFinishedSignal(int numSIP, int numRTP);
      void statusMessage (QString str);

	private:		
      int transSetStatusCode(int pos, int statusCode);      
      int transGetSizeAll(void);
      void clearInfoPacket(infoReportTrans_t *info);
      void clearTransState (transState_t *trans);

		QFutureWatcher<void> futureWatcher;
      QList<packet_t> listaPacotesAux;
		QList<transList_t> transLista;

		void bubbleSortPackets(int pos);
		void initSeqSourceRTP (sourceRTP_t *s, unsigned short seq, unsigned int ts, timeval ar);
		int updateSeqRTP(sourceRTP_t *s, unsigned short seq, timeval arrival, unsigned int ts);
		void jitterCalcPackets (sourceRTP_t *s, unsigned short seq, timeval ar, unsigned int ts);
   };

#endif // VOIPFIXTRANSACTIONS_H
