#include "voipfixwinpcap.h"
#include <QStringList>

VoIPFixWinpCap::VoIPFixWinpCap()
	{
	}

VoIPFixWinpCap::~VoIPFixWinpCap()
	{
	delete deviceList;
	pcap_freealldevs(alldevs);
	}

int VoIPFixWinpCap::startWinpCap()
   {
	alldevs = NULL;
	adhandle = NULL;
	deviceList = new QStringList();
	deviceList->clear();
   deviceOpened = false;
   fileOpened = false;
   closeStopDevice = false;
	openSequential = false;
	contFileSequential = 0;

	connect (this, SIGNAL(nFile()), this, SLOT(nextFileSequential()));
	return 0;
   }

QStringList * VoIPFixWinpCap::getDeviceList()
	{
	char errbuf[PCAP_ERRBUF_SIZE];
	pcap_if_t * alldevsAux;

	deviceList->clear();
	pcap_freealldevs(alldevs);
	alldevs = NULL;

#ifdef Q_OS_WIN
	if (pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL /* auth is not needed */, &alldevs, errbuf) != -1)
#endif
#ifdef Q_OS_UNIX
        if (pcap_findalldevs(&alldevs, errbuf) != -1)
#endif
		{
		if (alldevs != NULL)
			{
			alldevsAux = alldevs;
			do
				{
#ifdef Q_OS_WIN
            deviceList->append(alldevsAux->description);
#endif
#ifdef Q_OS_UNIX
                                deviceList->append(alldevsAux->name);
#endif
                                alldevsAux = alldevsAux->next;
				} while (alldevsAux != NULL);
			}
		}
	return deviceList;
	}

int VoIPFixWinpCap::openDevice (int deviceNumber, int inPromiscuousMode, char * filter)
	{
	pcap_if_t *d;
	u_int netmask;
   struct bpf_program fcode;

	char errbuf[PCAP_ERRBUF_SIZE];
	int i;

   if (deviceOpened == false)
      {
      if (alldevs != NULL)
         {
         /* Jump to the selected adapter */
         for(d=alldevs, i=0; i< deviceNumber; d=d->next, i++);

         /* Open the device */
#ifdef Q_OS_WIN
         if ( (adhandle= pcap_open(d->name,        // name of the device
                                 65536,            // portion of the packet to capture
                                                   // 65536 guarantees that the whole packet will be captured on all the link layers
                                 inPromiscuousMode,    // promiscuous mode
                                 1,             // read timeout
                                 NULL,             // authentication on the remote machine
                                 errbuf            // error buffer
                                 ) ) == NULL)
#endif
#ifdef Q_OS_UNIX
         if ( (adhandle= pcap_open_live(d->name,        // name of the device
                                 65536,
                                 inPromiscuousMode,
                                 1,
                                 errbuf
                                 ) ) == NULL)
#endif
            {
            /* Free the device list */
            pcap_freealldevs(alldevs);
            alldevs = NULL;
            return -1;	//No conseguiu abrir o dispositivo
            }
         else
            {
//				if ((filter != NULL) && (filter[0]))
//					{
//					if(d->addresses != NULL)
//						{
//						  /* Retrieve the mask of the first address of the interface */
//#ifdef Q_OS_WIN
//						netmask=((struct sockaddr_in *)(d->addresses->netmask))->sin_addr.S_un.S_addr;
//#endif
//#ifdef Q_OS_UNIX
//						//netmask=(u_int)((struct sockaddr_in *)(d->addresses->netmask))->sin_addr.s_addr;
//						netmask=0xffffff;
//#endif
//						}
//					else
//						{
//						/* If the interface is without addresses we suppose to be in a C class network */
//						netmask=0xffffff;
//						}

//					//compile the filter
//					//ToDo - deal return
//               pcap_compile(adhandle, &fcode, filter, 1, netmask);

//					//set the filter
//					//ToDo - deal return
//					pcap_setfilter(adhandle, &fcode);
//					}

            if ((filter != NULL) && (filter[0]))
               {
               netmask=0xffffff;
               //compile the filter
               //ToDo - Deal return
               pcap_compile(adhandle, &fcode, filter, 1, netmask);

               //set the filter
               //ToDo - Deal return
               pcap_setfilter(adhandle, &fcode);
               }
            deviceOpened = true;
            closeStopDevice = false;

            pcap_freealldevs(alldevs);
            alldevs = NULL;

            start(QThread::TimeCriticalPriority);
				return 0;	//Device successful opened
            }
         }
      else
         {
			return -2;	//No device was selected
         }
      }
   else
      {
      return -3;
      }
	}

void VoIPFixWinpCap::closeDevice (void)
	{
   if (deviceOpened)
      {
      deviceOpened = false;
      closeStopDevice = true;
      }
	}

void VoIPFixWinpCap::run(void)
	{
	int res;
   struct pcap_pkthdr *header, *header_aux;
   const u_char *pkt_data;
   u_char *pkt_data_aux;

	/* Retrieve the packets */
   while((res = pcap_next_ex( adhandle, &header, &pkt_data)) >= 0)
		{
      if (closeStopDevice)
         {
         closeStopDevice = false;
         res = 0;
         pcap_close(adhandle);
         break;
         }

      if(res == 0)
			/* Timeout elapsed */
			continue;

      header_aux = (pcap_pkthdr*)malloc(sizeof(pcap_pkthdr));
      memcpy (header_aux, header, sizeof(pcap_pkthdr));
		//free (header);

      //packet received. emit signal
		//if (deviceOpened)
		//   {
		//   pkt_data_aux = (u_char*)pkt_data;
		//   emit (winPacket(header_aux, pkt_data_aux));
		//   }
		//else
         {
         pkt_data_aux = (u_char*)malloc(header_aux->caplen);
         memcpy ((u_char*)pkt_data_aux, pkt_data, header_aux->caplen);
         emit (winPacket(header_aux, pkt_data_aux));
         }
      }

   if (res == -1)
      {
		//qDebug("Capture troubles.");
      }
   else if (res == -2)
      {
		closeFile();
		if (openSequential)
			{
			emit (nFile());
			}
		else
			{
			emit (winEOF());
			}
      }
	}

int VoIPFixWinpCap::openFile(char * fileName, char * filter)
	{
	char errbuf[PCAP_ERRBUF_SIZE];
	char source[PCAP_BUF_SIZE];
	u_int netmask;
   struct bpf_program fcode;

   if (fileOpened == false)
      {
#ifdef Q_OS_WIN
      /* Create the source string according to the new WinPcap syntax */
      if ( pcap_createsrcstr( source,         // variable that will keep the source string
                              PCAP_SRC_FILE,  // we want to open a file
                              NULL,           // remote host
                              NULL,           // port on the remote host
                              fileName,        // name of the file we want to open
                              errbuf          // error buffer
                              ) != 0)
         {
         return -1;
         }

      /* Open the capture file */
      if ( (adhandle= pcap_open(source,         // name of the device
                              65536,          // portion of the packet to capture
                              // 65536 guarantees that the whole packet will be captured on all the link layers
                              PCAP_OPENFLAG_PROMISCUOUS,     // promiscuous mode
                              1000,              // read timeout
                              NULL,              // authentication on the remote machine
                              errbuf         // error buffer
                              ) ) == NULL)
#endif

#ifdef Q_OS_UNIX
      /* Open the capture file */
      if ( (adhandle= pcap_open_offline(fileName,         // name of the device
                              errbuf         // error buffer
                              ) ) == NULL)
#endif
         {
         return -2;
         }
      else
         {
			if ((filter != NULL) && (filter[0]))
				{
				netmask=0xffffff;
				//compile the filter
				//ToDo - Deal return
				pcap_compile(adhandle, &fcode, filter, 1, netmask);

				//set the filter
				//ToDo - Deal return
				pcap_setfilter(adhandle, &fcode);
				}

         pcap_freealldevs(alldevs);
         alldevs = NULL;

         fileOpened = true;
         closeStopDevice = false;
         start(QThread::TimeCriticalPriority);
			return 0;	//Device successful opened
         }
      }
   else
      {
      return -3;
      }
	}

int VoIPFixWinpCap::openFileSequencial(QStringList files, char * filter)
	{
	QString strAux;
	fileNames = files;
	if (fileNames.size() >= 1)
		{
		openSequential = true;
		strAux = fileNames.at(0);
		contFileSequential = 0;
		filterSet = filter;
		return (openFile(strAux.toLatin1().data(), filter));
		}
	else
		{
		return -4;
		}
	}

void VoIPFixWinpCap::nextFileSequential(void)
	{
	QString strAux;
	if (++contFileSequential < fileNames.size())
		{
		strAux = fileNames.at(contFileSequential);
		openFile(strAux.toLatin1().data(), filterSet);
		}
	else
		{
		openSequential = false;
		contFileSequential = 0;
		emit (winEOF());
		}
	}

void VoIPFixWinpCap::closeFile(void)
   {
   pcap_close(adhandle);
   adhandle = NULL;
   fileOpened = false;
   }

int VoIPFixWinpCap::saveFile (char * fileName)
   {
   int res;

   res = -1;
   fileOpened = false;
   if (adhandle != NULL)
      {
      /* Open the dump file */
      dumpfile = pcap_dump_open(adhandle, fileName);

      if(dumpfile==NULL)
         {
         res = -2;
         }
      else
         {
         fileOpened = true;
         res = 0;
         }
      }

   return res;
   }

int VoIPFixWinpCap::packetDumpFile (const struct pcap_pkthdr *header, const u_char *pkt_data)
   {
   if (dumpfile != NULL)
      {
      /* save the packet on the dump file */
      pcap_dump((u_char *)dumpfile, header, pkt_data);
      return 0;
      }
   else
      {
      return -1;
      }
   }

int VoIPFixWinpCap::closeDumpFile (void)
   {
   if (dumpfile != NULL)
      {
      pcap_dump_close (dumpfile);
      dumpfile = NULL;
      fileOpened = false;
      return 0;
      }
   else
      {
      return -1;
      }
   }


