#include "windowestatisticas.h"
#include <time.h>

WindowStatistics::WindowStatistics()
	{
	setupUi(this);

	setMinimumSize(900, 600);
	setWindowTitle(tr("Estatsticas de Transaes"));
	setAttribute(Qt::WA_DeleteOnClose);
	srand(time(NULL));
	}

void WindowStatistics::closeEvent(QCloseEvent *event)
	{
   emit closingStatistics(true);
	}

void WindowStatistics::setInfoTrans (QList <transList_t> tList)
	{
	transLista = tList;
	tabWidget = new QTabWidget;
	tabWidget->addTab(new GeralTab(transLista),					tr("Nmero de Transaes"));
	tabWidget->addTab(new InfoChamadasTab(transLista),			tr("Estado de Chamadas"));
	tabWidget->addTab(new UserFromChamadasTab(transLista),	tr("Chamadas do Usurio"));
	tabWidget->addTab(new UserToChamadasTab(transLista),		tr("Chamadas para o Usurio"));
	tabWidget->addTab(new IPFromChamadasTab(transLista),		tr("Chamadas do IP"));
	tabWidget->addTab(new IPToChamadasTab(transLista),			tr("Chamadas para o IP"));
	tabWidget->addTab(new InfoRegistroTab(transLista),			tr("Estado de Registro"));
	tabWidget->addTab(new UserFromRegistroTab(transLista),	tr("Registro do Usurio"));
	tabWidget->addTab(new IPFromRegistroTab(transLista),		tr("Registro do IP"));
	tabWidget->addTab(new IPToRegistroTab(transLista),			tr("Registro para o IP"));
	setCentralWidget(tabWidget);
	}

GeralTab::GeralTab(QList <transList_t> tList, QWidget *parent)
	{
	int j;
	int listSize;
	int numRegister;
	int numInvite;
	int numOptions;
	int numNotify;
	int numOther;

	numRegister = 0;
	numInvite = 0;
	numOptions = 0;
	numNotify = 0;
	numOther = 0;

	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "INVITE", Qt::CaseInsensitive) == 0)
			{
			numInvite++;
			}
		else if (QString::compare(tList.at(j).sipMethod, "REGISTER", Qt::CaseInsensitive) == 0)
			{
			numRegister++;
			}
		else if (QString::compare(tList.at(j).sipMethod, "OPTIONS", Qt::CaseInsensitive) == 0)
			{
			numOptions++;
			}
		else if (QString::compare(tList.at(j).sipMethod, "NOTIFY", Qt::CaseInsensitive) == 0)
			{
			numNotify++;
			}
		else
			{
			numOther++;
			}
		}

	QLabel *label = new QLabel(tr("Tipos de Transaes"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, 580)));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 580);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	bar = new barGraph();
	bar->drawBar(0, numInvite, listSize, "INVITE", QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(1, numRegister, listSize, "REGISTER", QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(2, numOptions, listSize, "OPTIONS", QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(3, numNotify, listSize, "NOTIFY", QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(4, numOther, listSize, tr("Outras"), QColor(Qt::blue));
	sceneGeral->addItem(bar);
	}

InfoChamadasTab::InfoChamadasTab(QList <transList_t> tList, QWidget *parent)
	{
	int j;
	int listSize;

	int terminated;
	int completed;
	int busy;
	int canceled;
	int refused;
	int not_authorized;
	int media_no_supported;
	int numOther;
	int numTotal;
	numTotal = 0;

	terminated = 0;
	completed = 0;
	busy = 0;
	canceled = 0;
	refused = 0;
	not_authorized = 0;
	media_no_supported = 0;
	numOther = 0;

	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "INVITE", Qt::CaseInsensitive) == 0)
			{
			switch (tList.at(j).state)
				{
				case TRANS_STATUS_BYE:
					{
					terminated++;
					break;
					}
				case 200:
				case 202:
					{
					completed++;
					break;
					}
				case 486:
					{
					busy++;
					break;
					}
				case TRANS_STATUS_CANCEL:
					{
					canceled++;
					break;
					}
				case 403:
					{
					refused++;
					break;
					}
				case 401:
				case 407:
					{
					not_authorized++;
					break;
					}
				case 415:
					{
					media_no_supported++;
					break;
					}
				default:
					{
					numOther++;
					}
				}
			numTotal++;
			}
		}

	QLabel *label = new QLabel(tr("Estado de Chamadas"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, 580)));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 580);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	bar = new barGraph();
	bar->drawBar(0, terminated, numTotal, tr("Terminada"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(1, completed, numTotal, tr("Completada"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(2, busy, numTotal, tr("Ocupada"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(3, canceled, numTotal, tr("Cancelada"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(4, refused, numTotal, tr("Negada"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(5, not_authorized, numTotal, tr("No Autorizada"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(6, media_no_supported, numTotal, tr("Mdia no sup."), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(7, numOther, numTotal, tr("Outras"), QColor(Qt::blue));
	sceneGeral->addItem(bar);
	}

UserFromChamadasTab::UserFromChamadasTab(QList <transList_t> tList, QWidget *parent)
	{
	int j, k;
	int listSize;
	int estSize;
	QList <estUser_t> estUser;
	estUser_t estUserAux;
	bool find;
	int numTotal;
	numTotal = 0;

	estUser.clear();
	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "INVITE", Qt::CaseInsensitive) == 0)
			{
			estSize = estUser.size();
			find = false;
			for (k = 0; k < estSize; k++)
				{
				if (QString::compare(tList.at(j).userFrom, estUser.at(k).user, Qt::CaseInsensitive) == 0)
					{
					find = true;
					break;
					}
				}
			if (find)
				{
				estUserAux = estUser.at(k);
				estUserAux.numOccurrence++;
				estUser.replace(k, estUserAux);
				}
			else
				{
				estUserAux.user = tList.at(j).userFrom;
				estUserAux.numOccurrence = 1;
				estUser.append(estUserAux);
				}
			numTotal++;
			}
		}

	QLabel *label = new QLabel(tr("Nmero de Chamadas do Usurio"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	estSize = estUser.size();
	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, estSize * (BAR_HEIGHT + BAR_SPACE))));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 500);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));
	viewGeral->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	viewGeral->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	quickSortEstUser(&estUser, 0, estUser.size()-1);
	for (k = 0; k < estSize; k++)
		{
		bar = new barGraph();
		bar->drawBar(k, estUser.at(k).numOccurrence, numTotal, estUser.at(k).user, QColor(Qt::blue));
		sceneGeral->addItem(bar);
		}
	}

UserToChamadasTab::UserToChamadasTab(QList <transList_t> tList, QWidget *parent)
	{
	int j, k;
	int listSize;
	int estSize;
	QList <estUser_t> estUser;
	estUser_t estUserAux;
	bool find;
	int numTotal;
	numTotal = 0;

	estUser.clear();
	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "INVITE", Qt::CaseInsensitive) == 0)
			{
			estSize = estUser.size();
			find = false;
			for (k = 0; k < estSize; k++)
				{
				if (QString::compare(tList.at(j).userTo, estUser.at(k).user, Qt::CaseInsensitive) == 0)
					{
					find = true;
					break;
					}
				}
			if (find)
				{
				estUserAux = estUser.at(k);
				estUserAux.numOccurrence++;
				estUser.replace(k, estUserAux);
				}
			else
				{
				estUserAux.user = tList.at(j).userTo;
				estUserAux.numOccurrence = 1;
				estUser.append(estUserAux);
				}
			numTotal++;
			}
		}

	QLabel *label = new QLabel(tr("Nmero de Chamadas para o Usurio"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	estSize = estUser.size();
	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, estSize * (BAR_HEIGHT + BAR_SPACE))));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 500);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));
	viewGeral->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	viewGeral->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	quickSortEstUser(&estUser, 0, estUser.size()-1);
	for (k = 0; k < estSize; k++)
		{
		bar = new barGraph();
		bar->drawBar(k, estUser.at(k).numOccurrence, numTotal, estUser.at(k).user, QColor(Qt::blue));
		sceneGeral->addItem(bar);
		}
	}

IPFromChamadasTab::IPFromChamadasTab(QList <transList_t> tList, QWidget *parent)
	{
	int j, k;
	int listSize;
	int estSize;
	QList <estIP_t> estIP;
	estIP_t estIPAux;
	bool find;
	int numTotal;
	numTotal = 0;
	QString strAux;

	estIP.clear();
	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "INVITE", Qt::CaseInsensitive) == 0)
			{
			estSize = estIP.size();
			find = false;
			for (k = 0; k < estSize; k++)
				{
				if (tList.at(j).ipFrom.ipdw == estIP.at(k).ip.ipdw)
					{
					find = true;
					break;
					}
				}
			if (find)
				{
				estIPAux = estIP.at(k);
				estIPAux.numOccurrence++;
				estIP.replace(k, estIPAux);
				}
			else
				{
				estIPAux.ip.ipdw = tList.at(j).ipFrom.ipdw;
				estIPAux.numOccurrence = 1;
				estIP.append(estIPAux);
				}
			numTotal++;
			}
		}

	QLabel *label = new QLabel(tr("Nmero de Chamadas do Endereo"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	estSize = estIP.size();
	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, estSize * (BAR_HEIGHT + BAR_SPACE))));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 500);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));
	viewGeral->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	viewGeral->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	quickSortEstIP(&estIP, 0, estIP.size()-1);
	for (k = 0; k < estSize; k++)
		{
		bar = new barGraph();
		bar->drawBar(k, estIP.at(k).numOccurrence, numTotal, strAux.sprintf("%i.%i.%i.%i", estIP.at(k).ip.ip[0],
		estIP.at(k).ip.ip[1], estIP.at(k).ip.ip[2], estIP.at(k).ip.ip[3]), QColor(Qt::blue));
		sceneGeral->addItem(bar);
		}
	}

IPToChamadasTab::IPToChamadasTab(QList <transList_t> tList, QWidget *parent)
	{
	int j, k;
	int listSize;
	int estSize;
	QList <estIP_t> estIP;
	estIP_t estIPAux;
	bool find;
	int numTotal;
	QString strAux;
	numTotal = 0;

	estIP.clear();
	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "INVITE", Qt::CaseInsensitive) == 0)
			{
			estSize = estIP.size();
			find = false;
			for (k = 0; k < estSize; k++)
				{
				if (tList.at(j).ipTo.ipdw == estIP.at(k).ip.ipdw)
					{
					find = true;
					break;
					}
				}
			if (find)
				{
				estIPAux = estIP.at(k);
				estIPAux.numOccurrence++;
				estIP.replace(k, estIPAux);
				}
			else
				{
				estIPAux.ip.ipdw = tList.at(j).ipTo.ipdw;
				estIPAux.numOccurrence = 1;
				estIP.append(estIPAux);
				}
			numTotal++;
			}
		}

	QLabel *label = new QLabel(tr("Nmero de Chamadas para o Endereo"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	estSize = estIP.size();
	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, estSize * (BAR_HEIGHT + BAR_SPACE))));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 500);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));
	viewGeral->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	viewGeral->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	quickSortEstIP(&estIP, 0, estIP.size()-1);
	for (k = 0; k < estSize; k++)
		{
		bar = new barGraph();
		bar->drawBar(k, estIP.at(k).numOccurrence, numTotal, strAux.sprintf("%i.%i.%i.%i", estIP.at(k).ip.ip[0],
		estIP.at(k).ip.ip[1], estIP.at(k).ip.ip[2], estIP.at(k).ip.ip[3]), QColor(Qt::blue));
		sceneGeral->addItem(bar);
		}
	}

InfoRegistroTab::InfoRegistroTab(QList <transList_t> tList, QWidget *parent)
	{
	int j;
	int listSize;

	int completed;
	int refused;
	int not_authorized;
	int not_found;
	int interval_too_brief;
	int numTotal;
	numTotal = 0;

	completed = 0;
	refused = 0;
	not_authorized = 0;
	not_found = 0;
	interval_too_brief = 0;

	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "REGISTER", Qt::CaseInsensitive) == 0)
			{
			switch (tList.at(j).state)
				{
				case 200:
				case 202:
					{
					completed++;
					break;
					}
				case 403:
					{
					refused++;
					break;
					}
				case 401:
				case 407:
					{
					not_authorized++;
					break;
					}
				case 404:
					{
					not_found++;
					break;
					}
				case 423:
					{
					interval_too_brief++;
					break;
					}
				}
			numTotal++;
			}
		}

	QLabel *label = new QLabel(tr("Estado de Registros"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, 580)));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 580);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	bar = new barGraph();
	bar->drawBar(0, completed, numTotal, tr("Completada"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(1, refused, numTotal, tr("Negada"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(2, not_authorized, numTotal, tr("No Autorizada"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(3, not_found, numTotal, tr("No Encontrado"), QColor(Qt::blue));
	sceneGeral->addItem(bar);

	bar = new barGraph();
	bar->drawBar(4, interval_too_brief, numTotal, tr("Interv. Curto"), QColor(Qt::blue));
	sceneGeral->addItem(bar);
	}

UserFromRegistroTab::UserFromRegistroTab(QList <transList_t> tList, QWidget *parent)
	{
	int j, k;
	int listSize;
	int estSize;
	QList <estUser_t> estUser;
	estUser_t estUserAux;
	bool find;
	int numTotal;
	numTotal = 0;

	estUser.clear();
	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "REGISTER", Qt::CaseInsensitive) == 0)
			{
			estSize = estUser.size();
			find = false;
			for (k = 0; k < estSize; k++)
				{
				if (QString::compare(tList.at(j).userFrom, estUser.at(k).user, Qt::CaseInsensitive) == 0)
					{
					find = true;
					break;
					}
				}
			if (find)
				{
				estUserAux = estUser.at(k);
				estUserAux.numOccurrence++;
				estUser.replace(k, estUserAux);
				}
			else
				{
				estUserAux.user = tList.at(j).userFrom;
				estUserAux.numOccurrence = 1;
				estUser.append(estUserAux);
				}
			numTotal++;
			}
		}

	QLabel *label = new QLabel(tr("Nmero de Registros do Usurio"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	estSize = estUser.size();
	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, estSize * (BAR_HEIGHT + BAR_SPACE))));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 580);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	quickSortEstUser(&estUser, 0, estUser.size()-1);
	for (k = 0; k < estSize; k++)
		{
		bar = new barGraph();
		bar->drawBar(k, estUser.at(k).numOccurrence, numTotal, estUser.at(k).user, QColor(Qt::blue));
		sceneGeral->addItem(bar);
		}
	}


IPFromRegistroTab::IPFromRegistroTab(QList <transList_t> tList, QWidget *parent)
	{
	int j, k;
	int listSize;
	int estSize;
	QList <estIP_t> estIP;
	estIP_t estIPAux;
	bool find;
	int numTotal;
	QString strAux;
	numTotal = 0;

	estIP.clear();
	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "REGISTER", Qt::CaseInsensitive) == 0)
			{
			estSize = estIP.size();
			find = false;
			for (k = 0; k < estSize; k++)
				{
				if (tList.at(j).ipFrom.ipdw == estIP.at(k).ip.ipdw)
					{
					find = true;
					break;
					}
				}
			if (find)
				{
				estIPAux = estIP.at(k);
				estIPAux.numOccurrence++;
				estIP.replace(k, estIPAux);
				}
			else
				{
				estIPAux.ip.ipdw = tList.at(j).ipFrom.ipdw;
				estIPAux.numOccurrence = 1;
				estIP.append(estIPAux);
				}
			numTotal++;
			}
		}

	QLabel *label = new QLabel(tr("Nmero de Registros do Endereo"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	estSize = estIP.size();
	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, estSize * (BAR_HEIGHT + BAR_SPACE))));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 580);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	quickSortEstIP(&estIP, 0, estIP.size()-1);
	for (k = 0; k < estSize; k++)
		{
		bar = new barGraph();
		bar->drawBar(k, estIP.at(k).numOccurrence, numTotal, strAux.sprintf("%i.%i.%i.%i", estIP.at(k).ip.ip[0],
		estIP.at(k).ip.ip[1], estIP.at(k).ip.ip[2], estIP.at(k).ip.ip[3]), QColor(Qt::blue));
		sceneGeral->addItem(bar);
		}
	}

IPToRegistroTab::IPToRegistroTab(QList <transList_t> tList, QWidget *parent)
	{
	int j, k;
	int listSize;
	int estSize;
	QList <estIP_t> estIP;
	estIP_t estIPAux;
	bool find;
	int numTotal;
	QString strAux;

	numTotal = 0;

	estIP.clear();
	listSize = tList.size();
	for (j = 0; j < listSize; j++)
		{
		if (QString::compare(tList.at(j).sipMethod, "REGISTER", Qt::CaseInsensitive) == 0)
			{
			estSize = estIP.size();
			find = false;
			for (k = 0; k < estSize; k++)
				{
				if (tList.at(j).ipTo.ipdw == estIP.at(k).ip.ipdw)
					{
					find = true;
					break;
					}
				}
			if (find)
				{
				estIPAux = estIP.at(k);
				estIPAux.numOccurrence++;
				estIP.replace(k, estIPAux);
				}
			else
				{
				estIPAux.ip.ipdw = tList.at(j).ipTo.ipdw;
				estIPAux.numOccurrence = 1;
				estIP.append(estIPAux);
				}
			numTotal++;
			}
		}

	QLabel *label = new QLabel(tr("Nmero de Registros para Endereo"));
	label->setFont(QFont("Courier New", 14, QFont::Bold, false));

	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(label, 0, 0, Qt::AlignCenter);
	layout->setSpacing(50);

	estSize = estIP.size();
	sceneGeral = new QGraphicsScene();
	sceneGeral->setSceneRect((QRectF(0, 0, 890, estSize * (BAR_HEIGHT + BAR_SPACE))));
	viewGeral = new QGraphicsView(this);
	viewGeral->setScene(sceneGeral);
	viewGeral->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
	viewGeral->setMinimumSize(890, 580);
	viewGeral->centerOn(0, 0);
	viewGeral->setFrameShape(QFrame::NoFrame);
	viewGeral->setBackgroundBrush(QBrush(Qt::white));

	layout->addWidget(viewGeral, 1, 0, Qt::AlignTop | Qt::AlignLeft);
	layout->setColumnStretch(1, 20);
	layout->setRowStretch(2, 20);

	quickSortEstIP(&estIP, 0, estIP.size()-1);
	for (k = 0; k < estSize; k++)
		{
		bar = new barGraph();
		bar->drawBar(k, estIP.at(k).numOccurrence, numTotal, strAux.sprintf("%i.%i.%i.%i", estIP.at(k).ip.ip[0],
		estIP.at(k).ip.ip[1], estIP.at(k).ip.ip[2], estIP.at(k).ip.ip[3]), QColor(Qt::blue));
		sceneGeral->addItem(bar);
		}
	}

void quickSortEstUser(QList <estUser_t> *eUser, int p, int r)
	{
	int q;
	if (p < r)
		{
		q = partitionEstUser(eUser, p, r);
		quickSortEstUser(eUser, p, q - 1);
		quickSortEstUser(eUser, q + 1, r);
		}
	}

int partitionEstUser(QList <estUser_t> *eUser, int p, int r)
	{
	int i, j;
	estUser_t estUserAux;

	i = p;

	for (j = p + 1; j <= r; ++j)
		{
		if (eUser->at(j).numOccurrence > eUser->at(p).numOccurrence)
			{
			++i;
			estUserAux.numOccurrence = eUser->at(j).numOccurrence;
			estUserAux.user = eUser->at(j).user;
			eUser->replace(j, eUser->at(i));
			eUser->replace(i, estUserAux);
			}
		}

	estUserAux.numOccurrence = eUser->at(p).numOccurrence;
	estUserAux.user = eUser->at(p).user;
	eUser->replace(p, eUser->at(i));
	eUser->replace(i, estUserAux);

	return (i);
	}

void quickSortEstIP(QList <estIP_t> *eUser, int p, int r)
	{
	int q;
	if (p < r)
		{
		q = partitionEstIP(eUser, p, r);
		quickSortEstIP(eUser, p, q - 1);
		quickSortEstIP(eUser, q + 1, r);
		}
	}

int partitionEstIP(QList <estIP_t> *eUser, int p, int r)
	{
	int i, j;
	estIP_t estIPAux;

	i = p;

	for (j = p + 1; j <= r; ++j)
		{
		if (eUser->at(j).numOccurrence > eUser->at(p).numOccurrence)
			{
			++i;
			estIPAux.numOccurrence = eUser->at(j).numOccurrence;
			estIPAux.ip = eUser->at(j).ip;
			eUser->replace(j, eUser->at(i));
			eUser->replace(i, estIPAux);
			}
		}

	estIPAux.numOccurrence = eUser->at(p).numOccurrence;
	estIPAux.ip = eUser->at(p).ip;
	eUser->replace(p, eUser->at(i));
	eUser->replace(i, estIPAux);

	return (i);
	}
