#include "windowpackets.h"
#include "osip_utils.h"

windowPackets::windowPackets()
	{
	setMinimumSize(950, 600);
	setWindowTitle(tr("Pacotes"));
	setAttribute(Qt::WA_DeleteOnClose);

	packetsModel = new QSortFilterProxyModel;
	packetsModel->setDynamicSortFilter(true);

	model = createMailModel(this);
	setSourceModel(model);

	packetsView = new QTreeView();	
	packetsView->setMinimumSize(950, 400);
	packetsView->setRootIsDecorated(false);
	packetsView->setAlternatingRowColors(true);
	packetsView->setModel(packetsModel);
	packetsView->setSortingEnabled(false);
	packetsView->sortByColumn(0, Qt::AscendingOrder);
	packetsView->setSelectionMode(QAbstractItemView::SingleSelection);
	packetsView->setSelectionBehavior(QAbstractItemView::SelectRows);
	packetsView->setFont(QFont("Courier New"));

	packetsView->setColumnWidth(0, 55);
	packetsView->setColumnWidth(1, 120);
	packetsView->setColumnWidth(2, 160);
	packetsView->setColumnWidth(3, 160);
	packetsView->setColumnWidth(4, 70);

	connect (packetsView, SIGNAL(clicked(QModelIndex)), this, SLOT(tClicked(QModelIndex)));
	numPackets = 0;

	QWidget *widget = new QWidget();
	QVBoxLayout *layout = new QVBoxLayout ();
	layout->addWidget(packetsView, 0);

	infoView = new QTreeWidget ();
	infoView->setMinimumSize(830, 200);
	infoView->setColumnCount(1);
	infoView->setHeaderLabel(tr("Detalhes do pacote"));
	infoView->header()->setResizeMode(0, QHeaderView::Stretch);
	infoView->setFont(QFont("Courier New"));
	infoView->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	infoView->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	layout->addWidget(infoView, 0);

	layout->setSpacing(0);

	widget->setLayout(layout);
	setCentralWidget(widget);
	}

QTreeWidgetItem * windowPackets::addParent(QTreeWidgetItem *father, QString text)
	{
	QTreeWidgetItem *item;

	if (father == NULL)
		{
		item = new QTreeWidgetItem (infoView->invisibleRootItem());
		}
	else
		{
		item = new QTreeWidgetItem (father);
		}
	item->setText(0, text);
	return (item);
	}

void windowPackets::setInfoPackets(QList <infoPacketTrans_t> infoPacket, QList<packet_t> listP)
	{
	int listaSize, j;
	infoPacketTrans_t infoAux;
	time_t local_tv_sec;
	struct tm *ltime;
	char timestr[16];
	char tStart[30];
	QString strIpAux;

	listaSize = infoPacket.size();

	packetsList = listP;
	infoPacketAux = infoPacket;

	for (j = 0; j < listaSize; j++)
		{
		infoAux = infoPacket.at(j);

		model->insertRow(j);

		//packet number
		model->setData(model->index(j, 0), infoAux.numPacket + 1);
		model->itemFromIndex(model->index(j, 0))->setEditable(false);

		//packet time
		local_tv_sec = infoAux.time.tv_sec;
		ltime = localtime (&local_tv_sec);
		strftime( timestr, sizeof timestr, "%H:%M:%S", ltime);
		sprintf(tStart, "%s.%.6ld", timestr, infoAux.time.tv_usec);
		model->setData(model->index(j, 1), tStart);
		model->itemFromIndex(model->index(j, 1))->setEditable(false);
		model->itemFromIndex(model->index(j, 1))->setTextAlignment(Qt::AlignCenter);

		//source IP of the packet
		strIpAux.clear();
		strIpAux.sprintf("%i.%i.%i.%i:%i", infoAux.srcIP.ip[0], infoAux.srcIP.ip[1], infoAux.srcIP.ip[2], infoAux.srcIP.ip[3], infoAux.srcPort);
		model->setData(model->index(j, 2), strIpAux);
		model->itemFromIndex(model->index(j, 2))->setEditable(false);
		model->itemFromIndex(model->index(j, 2))->setTextAlignment(Qt::AlignCenter);

		//destination IP of the packet
		strIpAux.clear();
		strIpAux.sprintf("%i.%i.%i.%i:%i", infoAux.dstIP.ip[0], infoAux.dstIP.ip[1], infoAux.dstIP.ip[2], infoAux.dstIP.ip[3], infoAux.dstPort);
		model->setData(model->index(j, 3), strIpAux);
		model->itemFromIndex(model->index(j, 3))->setEditable(false);
		model->itemFromIndex(model->index(j, 3))->setTextAlignment(Qt::AlignCenter);

		//packet protocol
		model->setData(model->index(j, 4), infoAux.info);
		model->itemFromIndex(model->index(j, 4))->setEditable(false);
		model->itemFromIndex(model->index(j, 4))->setTextAlignment(Qt::AlignCenter);

		//packet information
		model->setData(model->index(j, 5), infoAux.coment);
		model->itemFromIndex(model->index(j, 5))->setEditable(false);
		}
	}

void windowPackets::clearPackets(void)
	{

	}

void windowPackets::contextMenuEvent(QContextMenuEvent *event)
	{

	}

void windowPackets::tClicked(QModelIndex modelIndex)
	{
	packet_t pacoteAux;
	QTreeWidgetItem *item;
	QTreeWidgetItem *itemHeader;
	QTreeWidgetItem *itemBody;
	QTreeWidgetItem *itemUrl;
	QString linha;
	char * strAux;
	int k, j;
	osip_via_t * viaAux;
	osip_from_t *fromAux;
	osip_uri *urlAux;
	osip_call_id_t *callIdAux;
	osip_cseq_t *cSeqAux;
	osip_authorization_t *authorizationAux;
	osip_www_authenticate_t *wwwAuthenticateAux;
	osip_content_type_t *contentTypeAux;
	osip_mime_version_t *mimeVersionAux;
	osip_allow_t *allowAux;
	bool hasAllow;
	osip_content_encoding_t *contentEncodingAux;
	osip_call_info_t *callInfoAux;
	osip_alert_info_t *alertInfoAux;
	osip_error_info_t *errorInfoAux;
	osip_accept_t *acceptAux;
	osip_accept_encoding_t *acceptEncodingAux;
	osip_accept_language_t *acceptLanguageAux;
	osip_authentication_info_t *AuthInfoAux;
	osip_proxy_authentication_info_t *proxyAuthInfoAux;
	osip_header_t * header;

	sdp_bandwidth_t *sdpBandwidthAux;
	sdp_time_descr_t *sdpTimeDescrAux;
	sdp_attribute_t *sdpAttributeAux;
	sdp_media_t *sdpMediaAux;
	osip_body *sipBodyAux;

	int pos = 0;
	osip_generic_param_t *u_param;

	pacoteAux = packetsList.at(infoPacketAux.at(modelIndex.row()).numPacket);

	infoView->clear();
	if (pacoteAux.infoRTP != NULL)
		{	// a RTP packet
		linha.sprintf("Protocol version: %i", pacoteAux.infoRTP->version);
		addParent(NULL, linha);

		if (pacoteAux.infoRTP->p)
			{
			linha.sprintf("Padding flag: true");
			}
		else
			{
			linha.sprintf("Padding flag: false");
			}
		addParent(NULL, linha);

		if (pacoteAux.infoRTP->x)
			{
			linha.sprintf("Header extension flag: true");
			}
		else
			{
			linha.sprintf("Header extension flag: false");
			}
		addParent(NULL, linha);

		linha.sprintf("CSRC count: %i", pacoteAux.infoRTP->cc);
		addParent(NULL, linha);

		if (pacoteAux.infoRTP->m)
			{
			linha.sprintf("Marker bit: true");
			}
		else
			{
			linha.sprintf("Marker bit: false");
			}
		addParent(NULL, linha);

		linha.sprintf("Payload type: %s", get_payload_type_str(pacoteAux.infoRTP->pt));
		addParent(NULL, linha);

		linha.sprintf("Sequence number: %i", pacoteAux.infoRTP->seq);
		addParent(NULL, linha);

		linha.sprintf("Timestamp: %i", pacoteAux.infoRTP->ts);
		addParent(NULL, linha);

		linha.sprintf("Synchronization source: 0x%x", pacoteAux.infoRTP->ssrc);
		addParent(NULL, linha);
		}
	else if (pacoteAux.sip_message != NULL)
		{	// a SIP message
		if (pacoteAux.sip_message->sip_method != NULL)
			{	//request message
			//request-line - *
			osip_uri_to_str(pacoteAux.sip_message->req_uri, &strAux);
			linha.sprintf("Request-Line: %s %s %s", pacoteAux.sip_message->sip_method, strAux, pacoteAux.sip_message->sip_version);
			item = addParent(NULL, linha);
			linha.sprintf("Method: %s", pacoteAux.sip_message->sip_method);
			addParent(item, linha);
			linha.sprintf("Request-Line: %s", strAux);
			item = addParent(item, linha);
			linha.sprintf("User: %s", pacoteAux.sip_message->req_uri->username);
			addParent(item, linha);
			linha.sprintf("Host: %s", pacoteAux.sip_message->req_uri->host);
			addParent(item, linha);
			osip_free(strAux);
			}
		else
			{	//answer message
			//status-line
			linha.sprintf("Status-Line: %s %i %s", pacoteAux.sip_message->sip_version, pacoteAux.sip_message->status_code, pacoteAux.sip_message->reason_phrase);
			item = addParent(NULL, linha);
			linha.sprintf("Status-Code: %i", pacoteAux.sip_message->status_code);
			addParent(item, linha);
			linha.sprintf("Reason Phrase: %s", pacoteAux.sip_message->reason_phrase);
			addParent(item, linha);
			}

		itemHeader = addParent(NULL, "Message Header");

		//"Via: " - *
		viaAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_via(pacoteAux.sip_message, k, &viaAux) < OSIP_SUCCESS) || (viaAux == NULL))
				break;

			osip_via_to_str(viaAux, &strAux);
			linha.sprintf("Via: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			linha.sprintf("Protocol: %s", osip_via_get_protocol(viaAux));
			addParent(item, linha);
			linha.sprintf("Host: %s", osip_via_get_host(viaAux));
			addParent(item, linha);
			linha.sprintf("Port: %s", osip_via_get_port(viaAux));
			addParent(item, linha);

			pos = 0;
			while (!osip_list_eol (&viaAux->via_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&viaAux->via_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(item, linha);
					}
				pos++;
				}
			k++;
			}while (viaAux != NULL);

		//"Record-Route:
		fromAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_record_route(pacoteAux.sip_message, k, &fromAux) < OSIP_SUCCESS) || (fromAux == NULL))
				break;

			osip_record_route_to_str(fromAux, &strAux);
			linha.sprintf("Record-Route: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			if (osip_from_get_displayname(fromAux) != NULL)
				{
				linha.sprintf("Display Name: %s", osip_from_get_displayname(fromAux));
				addParent(item, linha);
				}
			osip_uri_to_str(osip_from_get_url(fromAux), &strAux);
			linha.sprintf("Record-Route Address: %s", strAux);
			itemUrl = addParent(item, linha);
			osip_free(strAux);
			urlAux = osip_from_get_url(fromAux);
			linha.sprintf("Record-Route Address User: %s", osip_uri_get_username(urlAux));
			addParent(itemUrl, linha);
			linha.sprintf("Record-Route Address Host: %s", osip_uri_get_host(urlAux));
			addParent(itemUrl, linha);
			pos = 0;
			while (!osip_list_eol (&fromAux->url->url_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->url->url_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(itemUrl, linha);
					}
				pos++;
				}

			pos = 0;
			while (!osip_list_eol (&fromAux->gen_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->gen_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(item, linha);
					}
				pos++;
				}
			k++;
			}while (fromAux != NULL);

		//"Route:
		fromAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_route(pacoteAux.sip_message, k, &fromAux) < OSIP_SUCCESS) || (fromAux == NULL))
				break;

			osip_route_to_str(fromAux, &strAux);
			linha.sprintf("Route: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			if (osip_from_get_displayname(fromAux) != NULL)
				{
				linha.sprintf("Display Name: %s", osip_from_get_displayname(fromAux));
				addParent(item, linha);
				}
			osip_uri_to_str(osip_from_get_url(fromAux), &strAux);
			linha.sprintf("Route Address: %s", strAux);
			itemUrl = addParent(item, linha);
			osip_free(strAux);
			urlAux = osip_from_get_url(fromAux);
			linha.sprintf("Route Address User: %s", osip_uri_get_username(urlAux));
			addParent(itemUrl, linha);
			linha.sprintf("Route Address Host: %s", osip_uri_get_host(urlAux));
			addParent(itemUrl, linha);
			pos = 0;
			while (!osip_list_eol (&fromAux->url->url_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->url->url_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(itemUrl, linha);
					}
				pos++;
				}

			pos = 0;
			while (!osip_list_eol (&fromAux->gen_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->gen_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(item, linha);
					}
				pos++;
				}
			k++;
			}while (fromAux != NULL);

		//"From: " - *
		if ((fromAux = osip_message_get_from(pacoteAux.sip_message)) != NULL)
			{
			osip_from_to_str(fromAux, &strAux);
			linha.sprintf("From: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			if (osip_from_get_displayname(fromAux) != NULL)
				{
				linha.sprintf("Display Name: %s", osip_from_get_displayname(fromAux));
				addParent(item, linha);
				}
			osip_uri_to_str(osip_from_get_url(fromAux), &strAux);
			linha.sprintf("From Address: %s", strAux);
			itemUrl = addParent(item, linha);
			osip_free(strAux);
			urlAux = osip_from_get_url(fromAux);
			linha.sprintf("From Address User: %s", osip_uri_get_username(urlAux));
			addParent(itemUrl, linha);
			linha.sprintf("From Address Host: %s", osip_uri_get_host(urlAux));
			addParent(itemUrl, linha);
			pos = 0;
			while (!osip_list_eol (&fromAux->url->url_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->url->url_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(itemUrl, linha);
					}
				pos++;
				}

			pos = 0;
			while (!osip_list_eol (&fromAux->gen_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->gen_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(item, linha);
					}
				pos++;
				}
			}

		//"To: " - *
		if ((fromAux = osip_message_get_to(pacoteAux.sip_message)) != NULL)
			{
			osip_from_to_str(fromAux, &strAux);
			linha.sprintf("To: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			if (osip_from_get_displayname(fromAux) != NULL)
				{
				linha.sprintf("Display Name: %s", osip_from_get_displayname(fromAux));
				addParent(item, linha);
				}
			osip_uri_to_str(osip_from_get_url(fromAux), &strAux);
			linha.sprintf("To Address: %s", strAux);
			itemUrl = addParent(item, linha);
			osip_free(strAux);
			urlAux = osip_from_get_url(fromAux);
			linha.sprintf("To Address User: %s", osip_uri_get_username(urlAux));
			addParent(itemUrl, linha);
			linha.sprintf("To Address Host: %s", osip_uri_get_host(urlAux));
			addParent(itemUrl, linha);
			pos = 0;
			while (!osip_list_eol (&fromAux->url->url_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->url->url_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(itemUrl, linha);
					}
				pos++;
				}

			pos = 0;
			while (!osip_list_eol (&fromAux->gen_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->gen_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(item, linha);
					}
				pos++;
				}
			}

		//"Call-ID: "
		if ((callIdAux = osip_message_get_call_id(pacoteAux.sip_message)) != NULL)
			{
			osip_call_id_to_str(callIdAux, &strAux);
			linha.sprintf("Call-ID: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			}

		//"CSeq: " - *
		if ((cSeqAux = osip_message_get_cseq(pacoteAux.sip_message)) != NULL)
			{
			osip_cseq_to_str(cSeqAux, &strAux);
			linha.sprintf("CSeq: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);
			linha.sprintf("Sequence Number: %s", cSeqAux->number);
			addParent(item, linha);
			linha.sprintf("Method: %s", cSeqAux->method);
			addParent(item, linha);
			}

		//"Contact: "
		fromAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_contact(pacoteAux.sip_message, k, &fromAux) < OSIP_SUCCESS) || (fromAux == NULL))
				break;

			osip_route_to_str(fromAux, &strAux);
			linha.sprintf("Contact: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			if (osip_from_get_displayname(fromAux) != NULL)
				{
				linha.sprintf("Display Name: %s", osip_from_get_displayname(fromAux));
				addParent(item, linha);
				}
			osip_uri_to_str(osip_from_get_url(fromAux), &strAux);
			linha.sprintf("Contact Address: %s", strAux);
			itemUrl = addParent(item, linha);
			osip_free(strAux);
			urlAux = osip_from_get_url(fromAux);
			linha.sprintf("Contact Address User: %s", osip_uri_get_username(urlAux));
			addParent(itemUrl, linha);
			linha.sprintf("Contact Address Host: %s", osip_uri_get_host(urlAux));
			addParent(itemUrl, linha);
			pos = 0;
			while (!osip_list_eol (&fromAux->url->url_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->url->url_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(itemUrl, linha);
					}
				pos++;
				}

			pos = 0;
			while (!osip_list_eol (&fromAux->gen_params, pos))
				{
				u_param = (osip_generic_param_t *) osip_list_get (&fromAux->gen_params, pos);

				if (u_param->gname != NULL)
					{
					if (u_param->gvalue == NULL)
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gname);
						}
					else
						{
						linha.sprintf("%s: %s", u_param->gname, u_param->gvalue);
						}
					addParent(item, linha);
					}
				pos++;
				}
			k++;
			}while (fromAux != NULL);

		//"Authorization: "
		authorizationAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_authorization(pacoteAux.sip_message, k, &authorizationAux) < OSIP_SUCCESS) || (authorizationAux == NULL))
				break;

			osip_authorization_to_str(authorizationAux, &strAux);
			linha.sprintf("Authorization: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			if (authorizationAux->auth_type != NULL)
				{
				linha.sprintf("Authentication Type: %s", authorizationAux->auth_type);
				addParent(item, linha);
				}

			if (authorizationAux->username != NULL)
				{
				linha.sprintf("Username: %s", authorizationAux->username);
				addParent(item, linha);
				}

			if (authorizationAux->realm != NULL)
				{
				linha.sprintf("Realm: %s", authorizationAux->realm);
				addParent(item, linha);
				}

			if (authorizationAux->nonce != NULL)
				{
				linha.sprintf("Nonce: %s", authorizationAux->nonce);
				addParent(item, linha);
				}

			if (authorizationAux->uri != NULL)
				{
				linha.sprintf("Uri: %s", authorizationAux->uri);
				addParent(item, linha);
				}

			if (authorizationAux->response != NULL)
				{
				linha.sprintf("Response: %s", authorizationAux->response);
				addParent(item, linha);
				}

			if (authorizationAux->digest != NULL)
				{
				linha.sprintf("Digest: %s", authorizationAux->digest);
				addParent(item, linha);
				}

			if (authorizationAux->algorithm != NULL)
				{
				linha.sprintf("Algorithm: %s", authorizationAux->algorithm);
				addParent(item, linha);
				}

			if (authorizationAux->cnonce != NULL)
				{
				linha.sprintf("Cnonce: %s", authorizationAux->cnonce);
				addParent(item, linha);
				}

			if (authorizationAux->opaque != NULL)
				{
				linha.sprintf("Opaque: %s", authorizationAux->opaque);
				addParent(item, linha);
				}

			if (authorizationAux->message_qop != NULL)
				{
				linha.sprintf("Message qop: %s", authorizationAux->message_qop);
				addParent(item, linha);
				}

			if (authorizationAux->nonce_count != NULL)
				{
				linha.sprintf("Nonce count: %s", authorizationAux->nonce_count);
				addParent(item, linha);
				}

			if (authorizationAux->auth_param != NULL)
				{
				linha.sprintf("Auth param: %s", authorizationAux->auth_param);
				addParent(item, linha);
				}
			k++;
			}while (authorizationAux != NULL);

		//"WWW-Authenticate: "
		wwwAuthenticateAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_www_authenticate(pacoteAux.sip_message, k, &wwwAuthenticateAux) < OSIP_SUCCESS) || (wwwAuthenticateAux == NULL))
				break;

			osip_www_authenticate_to_str(wwwAuthenticateAux, &strAux);
			linha.sprintf("WWW-Authenticate: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			if (wwwAuthenticateAux->auth_type != NULL)
				{
				linha.sprintf("WWW-Authenticate Type: %s", wwwAuthenticateAux->auth_type);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->realm != NULL)
				{
				linha.sprintf("Realm: %s", wwwAuthenticateAux->realm);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->domain != NULL)
				{
				linha.sprintf("Domain: %s", wwwAuthenticateAux->domain);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->nonce != NULL)
				{
				linha.sprintf("Nonce: %s", wwwAuthenticateAux->nonce);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->opaque != NULL)
				{
				linha.sprintf("Opaque: %s", wwwAuthenticateAux->opaque);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->stale != NULL)
				{
				linha.sprintf("Stale: %s", wwwAuthenticateAux->stale);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->algorithm != NULL)
				{
				linha.sprintf("Algorithm: %s", wwwAuthenticateAux->algorithm);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->qop_options != NULL)
				{
				linha.sprintf("Qop Options: %s", wwwAuthenticateAux->qop_options);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->auth_param != NULL)
				{
				linha.sprintf("Auth param: %s", wwwAuthenticateAux->auth_param);
				addParent(item, linha);
				}
			k++;
			}while (wwwAuthenticateAux != NULL);

		//"Proxy-Authenticate: "
		wwwAuthenticateAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_proxy_authenticate(pacoteAux.sip_message, k, &wwwAuthenticateAux) < OSIP_SUCCESS) || (wwwAuthenticateAux == NULL))
				break;

			osip_proxy_authenticate_to_str(wwwAuthenticateAux, &strAux);
			linha.sprintf("Proxy-Authenticate: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			if (wwwAuthenticateAux->auth_type != NULL)
				{
				linha.sprintf("Proxy-Authenticate Type: %s", wwwAuthenticateAux->auth_type);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->realm != NULL)
				{
				linha.sprintf("Realm: %s", wwwAuthenticateAux->realm);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->domain != NULL)
				{
				linha.sprintf("Domain: %s", wwwAuthenticateAux->domain);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->nonce != NULL)
				{
				linha.sprintf("Nonce: %s", wwwAuthenticateAux->nonce);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->opaque != NULL)
				{
				linha.sprintf("Opaque: %s", wwwAuthenticateAux->opaque);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->stale != NULL)
				{
				linha.sprintf("Stale: %s", wwwAuthenticateAux->stale);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->algorithm != NULL)
				{
				linha.sprintf("Algorithm: %s", wwwAuthenticateAux->algorithm);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->qop_options != NULL)
				{
				linha.sprintf("Qop Options: %s", wwwAuthenticateAux->qop_options);
				addParent(item, linha);
				}

			if (wwwAuthenticateAux->auth_param != NULL)
				{
				linha.sprintf("Auth param: %s", wwwAuthenticateAux->auth_param);
				addParent(item, linha);
				}
			k++;
			}while (wwwAuthenticateAux != NULL);

		//"Proxy-Authorization: "
		authorizationAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_proxy_authorization(pacoteAux.sip_message, k, &authorizationAux) < OSIP_SUCCESS) || (authorizationAux == NULL))
				break;

			osip_proxy_authorization_to_str(authorizationAux, &strAux);
			linha.sprintf("Proxy-Authorization: %s", strAux);
			item = addParent(itemHeader, linha);
			osip_free(strAux);

			if (authorizationAux->auth_type != NULL)
				{
				linha.sprintf("Proxy-Authorization Type: %s", authorizationAux->auth_type);
				addParent(item, linha);
				}

			if (authorizationAux->username != NULL)
				{
				linha.sprintf("Username: %s", authorizationAux->username);
				addParent(item, linha);
				}

			if (authorizationAux->realm != NULL)
				{
				linha.sprintf("Realm: %s", authorizationAux->realm);
				addParent(item, linha);
				}

			if (authorizationAux->nonce != NULL)
				{
				linha.sprintf("Nonce: %s", authorizationAux->nonce);
				addParent(item, linha);
				}

			if (authorizationAux->uri != NULL)
				{
				linha.sprintf("Uri: %s", authorizationAux->uri);
				addParent(item, linha);
				}

			if (authorizationAux->response != NULL)
				{
				linha.sprintf("Response: %s", authorizationAux->response);
				addParent(item, linha);
				}

			if (authorizationAux->digest != NULL)
				{
				linha.sprintf("Digest: %s", authorizationAux->digest);
				addParent(item, linha);
				}

			if (authorizationAux->algorithm != NULL)
				{
				linha.sprintf("Algorithm: %s", authorizationAux->algorithm);
				addParent(item, linha);
				}

			if (authorizationAux->cnonce != NULL)
				{
				linha.sprintf("Cnonce: %s", authorizationAux->cnonce);
				addParent(item, linha);
				}

			if (authorizationAux->opaque != NULL)
				{
				linha.sprintf("Opaque: %s", authorizationAux->opaque);
				addParent(item, linha);
				}

			if (authorizationAux->message_qop != NULL)
				{
				linha.sprintf("Message qop: %s", authorizationAux->message_qop);
				addParent(item, linha);
				}

			if (authorizationAux->nonce_count != NULL)
				{
				linha.sprintf("Nonce count: %s", authorizationAux->nonce_count);
				addParent(item, linha);
				}

			if (authorizationAux->auth_param != NULL)
				{
				linha.sprintf("Auth param: %s", authorizationAux->auth_param);
				addParent(item, linha);
				}
			k++;
			}while (authorizationAux != NULL);

		//"Content-Type: "
		if ((contentTypeAux = osip_message_get_content_type(pacoteAux.sip_message)) != NULL)
			{
			osip_content_type_to_str(contentTypeAux, &strAux);
			linha.sprintf("Content-Type: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			}

		//"Mime-Version: "		
		if ((mimeVersionAux = osip_message_get_mime_version(pacoteAux.sip_message)) != NULL)
			{
			osip_mime_version_to_str(mimeVersionAux, &strAux);
			linha.sprintf("Mime-Version: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			}

		//"Allow: "
		k = 0;
		allowAux = NULL;

		hasAllow = false;
		do
			{
			if ((osip_message_get_allow(pacoteAux.sip_message, k, &allowAux) < OSIP_SUCCESS) || (allowAux == NULL))
				break;

			if (!hasAllow)
				{
				linha.sprintf("Allow: %s", allowAux->value);
				hasAllow = true;
				}
			else
				{
				linha.append(", ");
				linha.append(allowAux->value);
				}
			k++;
			}while (allowAux != NULL);
		if (hasAllow)
			addParent(itemHeader, linha);

		//"Content-Encoding: "
		k = 0;
		contentEncodingAux = NULL;
		do
			{
			if ((osip_message_get_content_encoding(pacoteAux.sip_message, k, &contentEncodingAux) < OSIP_SUCCESS) || (contentEncodingAux == NULL))
				break;
			osip_content_encoding_to_str(contentEncodingAux, &strAux);
			linha.sprintf("Content-Encoding: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			k++;
			}while (contentEncodingAux != NULL);

		//"Call-Info: "
		k = 0;
		callInfoAux = NULL;
		do
			{
			if ((osip_message_get_call_info(pacoteAux.sip_message, k, &callInfoAux) < OSIP_SUCCESS) || (callInfoAux == NULL))
				break;
			osip_call_info_to_str(callInfoAux, &strAux);
			linha.sprintf("Call-Info: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			k++;
			}while (callInfoAux != NULL);

		//"Alert-Info: "
		k = 0;
		alertInfoAux = NULL;
		do
			{
			if ((osip_message_get_alert_info(pacoteAux.sip_message, k, &alertInfoAux) < OSIP_SUCCESS) || (alertInfoAux == NULL))
				break;
			osip_alert_info_to_str(alertInfoAux, &strAux);
			linha.sprintf("Alert-Info: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			k++;
			}while (alertInfoAux != NULL);


		//"Error-Info: "
		k = 0;
		errorInfoAux = NULL;
		do
			{
			if ((osip_message_get_error_info(pacoteAux.sip_message, k, &errorInfoAux) < OSIP_SUCCESS) || (errorInfoAux == NULL))
				break;
			osip_error_info_to_str(errorInfoAux, &strAux);
			linha.sprintf("Error-Info: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			k++;
			}while (errorInfoAux != NULL);

		//"Accept: "
		k = 0;
		acceptAux = NULL;
		do
			{
			if ((osip_message_get_accept(pacoteAux.sip_message, k, &acceptAux) < OSIP_SUCCESS) || (acceptAux == NULL))
				break;
			osip_accept_to_str(acceptAux, &strAux);
			linha.sprintf("Accept: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			k++;
			}while (acceptAux != NULL);

		//"Accept-Encoding: "
		k = 0;
		acceptEncodingAux = NULL;
		do
			{
			if ((osip_message_get_accept_encoding(pacoteAux.sip_message, k, &acceptEncodingAux) < OSIP_SUCCESS) || (acceptEncodingAux == NULL))
				break;
			osip_accept_encoding_to_str(acceptEncodingAux, &strAux);
			linha.sprintf("Accept-Encoding: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			k++;
			}while (acceptEncodingAux != NULL);

		//"Accept-Language: "
		acceptLanguageAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_accept_language(pacoteAux.sip_message, k, &acceptLanguageAux) < OSIP_SUCCESS) || (acceptLanguageAux == NULL))
				break;
			osip_accept_language_to_str(acceptLanguageAux, &strAux);
			linha.sprintf("Accept-Language: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			k++;
			}while (acceptLanguageAux != NULL);

		//"Authentication-Info: "
		AuthInfoAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_authentication_info(pacoteAux.sip_message, k, &AuthInfoAux) < OSIP_SUCCESS) || (AuthInfoAux == NULL))
				break;
			osip_authentication_info_to_str(AuthInfoAux, &strAux);
			linha.sprintf("Authentication-Info: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			k++;
			}while (AuthInfoAux != NULL);

		//"Proxy-Authentication-Info: "
		proxyAuthInfoAux = NULL;
		k = 0;
		do
			{
			if ((osip_message_get_proxy_authentication_info(pacoteAux.sip_message, k, &proxyAuthInfoAux) < OSIP_SUCCESS) || (proxyAuthInfoAux == NULL))
				break;
			osip_proxy_authentication_info_to_str(proxyAuthInfoAux, &strAux);
			linha.sprintf("Proxy-Authentication-Info: %s", strAux);
			addParent(itemHeader, linha);
			osip_free(strAux);
			k++;
			}while (proxyAuthInfoAux != NULL);

		k = 0;
		header = NULL;
		do
			{
			if ((header = (osip_header_t *) osip_list_get (&pacoteAux.sip_message->headers, k)) == NULL)
				break;

			osip_header_to_str (header, &strAux);
			addParent(itemHeader, strAux);
			osip_free(strAux);
			k++;
			}while (header != NULL);

		if (pacoteAux.sdp_message != NULL)
			{
			itemBody = addParent(NULL, "Message Body");
			if ((contentTypeAux = osip_message_get_content_type(pacoteAux.sip_message)) != NULL)
				{
				osip_content_type_to_str(contentTypeAux, &strAux);
				if (strcmp(strAux, "application/sdp") == 0)
					{
					//v=	Protocol Version
					itemBody = addParent(itemBody, "Session Description Protocol");
					linha.sprintf("Protocol Version (v): %s", sdp_message_v_version_get(pacoteAux.sdp_message));
					addParent(itemBody, linha);

					//o=	Owner/Creator and session identifier
					linha.sprintf("Owner/Creator, session ID (s): %s %s %s %s %s %s", pacoteAux.sdp_message->o_username,
										pacoteAux.sdp_message->o_sess_id, pacoteAux.sdp_message->o_sess_version, pacoteAux.sdp_message->o_nettype,
										pacoteAux.sdp_message->o_addrtype, pacoteAux.sdp_message->o_addr);
					item = addParent(itemBody, linha);

					linha.sprintf("Username: %s", pacoteAux.sdp_message->o_username);
					addParent(item, linha);

					linha.sprintf("Session ID: %s", pacoteAux.sdp_message->o_sess_id);
					addParent(item, linha);

					linha.sprintf("Session version: %s", pacoteAux.sdp_message->o_sess_version);
					addParent(item, linha);

					linha.sprintf("Network type: %s", pacoteAux.sdp_message->o_nettype);
					addParent(item, linha);

					linha.sprintf("Address type: %s", pacoteAux.sdp_message->o_addrtype);
					addParent(item, linha);

					linha.sprintf("Address: %s", pacoteAux.sdp_message->o_addr);
					addParent(item, linha);

					//s=	Session name
					linha.sprintf("Session name (s): %s", pacoteAux.sdp_message->s_name);
					addParent(itemBody, linha);

					//i=	Session information
					if (pacoteAux.sdp_message->i_info != NULL)
						{
						linha.sprintf("Session information (i): %s", pacoteAux.sdp_message->i_info);
						addParent(itemBody, linha);
						}

					//u=	URI
					if (pacoteAux.sdp_message->u_uri != NULL)
						{
						linha.sprintf("URI (u): %s", pacoteAux.sdp_message->u_uri);
						addParent(itemBody, linha);
						}

					//e=	Email address
					strAux = NULL;
					k = 0;
					do
						{
						if ((strAux = sdp_message_e_email_get(pacoteAux.sdp_message, k)) == NULL)
							break;

						linha.sprintf("Email address (e): %s", strAux);
						addParent(itemBody, linha);
						k++;
						}while (strAux != NULL);

					//p=	Phone number
					strAux = NULL;
					k = 0;
					do
						{
						if ((strAux = sdp_message_p_phone_get(pacoteAux.sdp_message, k)) == NULL)
							break;

						linha.sprintf("Phone number (p): %s", strAux);
						addParent(itemBody, linha);
						k++;
						}while (strAux != NULL);

					//c=	Connection information
					linha.sprintf("Connection information (c): %s %s %s %s %s", pacoteAux.sdp_message->c_connection->c_nettype,
									pacoteAux.sdp_message->c_connection->c_addrtype, pacoteAux.sdp_message->c_connection->c_addr,
									pacoteAux.sdp_message->c_connection->c_addr_multicast_ttl, pacoteAux.sdp_message->c_connection->c_addr_multicast_int);
					item = addParent(itemBody, linha);

					linha.sprintf("Network type: %s", pacoteAux.sdp_message->c_connection->c_nettype);
					addParent(item, linha);

					linha.sprintf("Address type: %s", pacoteAux.sdp_message->c_connection->c_addrtype);
					addParent(item, linha);

					linha.sprintf("Connection address: %s", pacoteAux.sdp_message->c_connection->c_addr);
					addParent(item, linha);

					if (pacoteAux.sdp_message->c_connection->c_addr_multicast_ttl != NULL)
						{
						linha.sprintf("Multicast TTL value: %s", pacoteAux.sdp_message->c_connection->c_addr_multicast_ttl);
						addParent(item, linha);
						}

					if (pacoteAux.sdp_message->c_connection->c_addr_multicast_int != NULL)
						{
						linha.sprintf("Multicast number: %s", pacoteAux.sdp_message->c_connection->c_addr_multicast_int);
						addParent(item, linha);
						}

					//b=	Bandwidth information
					sdpBandwidthAux = NULL;
					k = 0;
					do
						{
						if ((sdpBandwidthAux = sdp_message_bandwidth_get(pacoteAux.sdp_message, -1, k)) == NULL)
							break;

						linha.sprintf("Bandwidth information (b): %s:%s", sdpBandwidthAux->b_bwtype, sdpBandwidthAux->b_bandwidth);
						addParent(itemBody, linha);
						k++;
						}while (sdpBandwidthAux != NULL);

					//t=	Time session starts and stops
					sdpTimeDescrAux = (sdp_time_descr_t*) osip_list_get(&pacoteAux.sdp_message->t_descrs, 0);
					linha.sprintf("Time description (t) %s %s", sdpTimeDescrAux->t_start_time, sdpTimeDescrAux->t_stop_time);
					item = addParent(itemBody, linha);

					linha.sprintf("Start time: %s", sdpTimeDescrAux->t_start_time);
					addParent(item, linha);

					linha.sprintf("Stop time: %s", sdpTimeDescrAux->t_stop_time);
					addParent(item, linha);

					//z=	Time zone corrections
					if (pacoteAux.sdp_message->z_adjustments != NULL)
						{
						linha.sprintf("Time zone corrections (z): %s", pacoteAux.sdp_message->z_adjustments);
						addParent(itemBody, linha);
						}

					//k=	Encryption key
					if (pacoteAux.sdp_message->k_key != NULL)
						{
						linha.sprintf("Encryption key (k): %s:%s", pacoteAux.sdp_message->k_key->k_keytype, pacoteAux.sdp_message->k_key->k_keydata);
						addParent(itemBody, linha);
						}

					//a=	Global attributes
					sdpAttributeAux = NULL;
					k = 0;
					do
						{
						if ((sdpAttributeAux = sdp_message_attribute_get(pacoteAux.sdp_message, -1, k)) == NULL)
							break;

						linha.sprintf("Global attributes (a): %s:%s", sdpAttributeAux->a_att_field, sdpAttributeAux->a_att_value);
						addParent(itemBody, linha);
						k++;
						} while (sdpAttributeAux != NULL);

					//m=	Media description
					sdpMediaAux = NULL;
					k = 0;
					do
						{
						if ((sdpMediaAux = (sdp_media_t*)osip_list_get(&pacoteAux.sdp_message->m_medias, k)) == NULL)
							break;

						linha.sprintf("Media description (m): %s %s %s ", sdpMediaAux->m_media, sdpMediaAux->m_port, sdpMediaAux->m_proto);
						j = 0;
						strAux = NULL;
						do
							{
							if ((strAux = (char*)osip_list_get(&sdpMediaAux->m_payloads, j)) == NULL)
								break;
							linha.append(strAux);
							linha.append(" ");
							j++;
							}while (strAux != NULL);
						item = addParent(itemBody, linha);

						linha.sprintf("Media type: %s", sdpMediaAux->m_media);
						addParent(item, linha);

						linha.sprintf("Media port: %s", sdpMediaAux->m_port);
						addParent(item, linha);

						linha.sprintf("Media protocol: %s", sdpMediaAux->m_proto);
						addParent(item, linha);

						j = 0;
						strAux = NULL;
						do
							{
							if ((strAux = (char*)osip_list_get(&sdpMediaAux->m_payloads, j)) == NULL)
								break;
							linha.sprintf("Media format: %s", strAux);
							addParent(item, linha);
							j++;
							}while (strAux != NULL);

						//a=	Media attributes
						sdpAttributeAux = NULL;
						j = 0;
						do
							{
							if ((sdpAttributeAux = sdp_message_attribute_get(pacoteAux.sdp_message, k, j)) == NULL)
								break;

							linha.sprintf("Media attributes (a): %s:%s", sdpAttributeAux->a_att_field, sdpAttributeAux->a_att_value);
							addParent(itemBody, linha);
							j++;
							} while (sdpAttributeAux != NULL);

						k++;
						} while (sdpMediaAux != NULL);
					}
				osip_free(strAux);
				}
			}
		else
			{
			if ((sipBodyAux = (osip_body_t*)osip_list_get(&pacoteAux.sip_message->bodies, 0)) != NULL)
				{
				itemBody = addParent(NULL, "Message Body");

				linha.sprintf("%s", sipBodyAux->body);
				addParent(itemBody, linha);
				}
			}
	  }
	}

void windowPackets::closeEvent(QCloseEvent *event)
	{
	emit closingPackest(true);
	}

QStandardItemModel * windowPackets::createMailModel(QObject *parent)
	{
	QStandardItemModel *modelAux = new QStandardItemModel(0, 6, parent);

	modelAux->setHeaderData(0, Qt::Horizontal, QObject::tr("Num"), Qt::DisplayRole);

	modelAux->setHeaderData(1, Qt::Horizontal, QObject::tr("Hora"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(1)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(2, Qt::Horizontal, QObject::tr("Origem"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(2)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(3, Qt::Horizontal, QObject::tr("Destino"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(3)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(4, Qt::Horizontal, QObject::tr("Protocolo"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(4)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(5, Qt::Horizontal, QObject::tr("Informao"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(5)->setTextAlignment(Qt::AlignCenter);

	return modelAux;
	}

void windowPackets::setSourceModel(QAbstractItemModel *modelAux)
	{
	packetsModel->setSourceModel(modelAux);
	}

void windowPackets::setPacketClicked (int numPacket)
	{
	int p, r, m;
	bool achou;

	//Binary search to find the line of the number of package
	r = infoPacketAux.size() - 1;
	p  = 0;
	achou = false;
	while (p <= r)
		{
		m = (p + r)/2;
		if (numPacket == infoPacketAux.at(m).numPacket)
			{
			achou = true;
			break;
			}
		if (numPacket < infoPacketAux.at(m).numPacket)
			r = m - 1;
		else
			p = m + 1;
		}

	if (achou == true)
		{
		packetsView->scrollTo(packetsView->model()->index(m, 0),QAbstractItemView::PositionAtCenter);
		packetsView->setCurrentIndex(packetsView->model()->index(m, 0));
		tClicked(packetsView->model()->index(m, 0));
		packetsView->setFocus();
		}
	}
