#include "windowreport.h"

WindowReport::WindowReport()
	{
	int k;
	setMinimumSize(950, 600);
	setWindowTitle(tr("Relatrio"));
	setAttribute(Qt::WA_DeleteOnClose);

	transModel = new QSortFilterProxyModel;
	transModel->setDynamicSortFilter(true);

	model = createMailModel(this);
	setSourceModel(model);

	transView = new QTreeView();
	transView->setMinimumSize(950, 400);
	transView->setRootIsDecorated(false);
	transView->setAlternatingRowColors(true);
	transView->setModel(transModel);
	transView->setSortingEnabled(false);
	transView->sortByColumn(0, Qt::AscendingOrder);
	transView->setSelectionMode(QAbstractItemView::SingleSelection);
	transView->setSelectionBehavior(QAbstractItemView::SelectRows);
	transView->setFont(QFont("Courier New"));

	k = 0;
	transView->setColumnWidth(k++, 55);			//numTrans
	transView->setColumnWidth(k++, 120);		//tStart
	transView->setColumnWidth(k++, 120);		//tStop
	transView->setColumnWidth(k++, 70);			//Type
	transView->setColumnWidth(k++, 90);			//Codec
	transView->setColumnWidth(k++, 120);		//IPFrom
	transView->setColumnWidth(k++, 140);		//userFrom
	transView->setColumnWidth(k++, 120);		//IPTo
	transView->setColumnWidth(k++, 140);		//userTo
	transView->setColumnWidth(k++, 160);		//state
	transView->setColumnWidth(k++, 120);		//duracao
	transView->setColumnWidth(k++, 160);		//meanJitterOrg
	transView->setColumnWidth(k++, 160);		//maxJitterOrg
	transView->setColumnWidth(k++, 160);		//packetLostPercOrg
	transView->setColumnWidth(k++, 160);		//sequenceErrorOrg
	transView->setColumnWidth(k++, 160);		//meanJitterRec
	transView->setColumnWidth(k++, 160);		//maxJitterRec
	transView->setColumnWidth(k++, 170);		//packetLostPercRec
	transView->setColumnWidth(k++, 170);		//sequenceErrorRec
	connect (transView, SIGNAL(clicked(QModelIndex)), this, SLOT(tClicked(QModelIndex)));

	infoView = new QTextEdit();
	infoView->setFont(QFont("Courier New"));
	infoView->setReadOnly(true);
	infoView->setMinimumSize(950, 200);
	infoView->clear();
	numTrans = 0;

	QWidget *widget = new QWidget();
	QVBoxLayout *layout = new QVBoxLayout ();
	layout->addWidget(transView, 0);
	layout->addWidget(infoView, 0);

	layout->setSpacing(0);

	widget->setLayout(layout);
	setCentralWidget(widget);
	}

void WindowReport::closeEvent(QCloseEvent *event)
	{
	emit closingReport(true);
	}

QStandardItemModel * WindowReport::createMailModel(QObject *parent)
	{
	int k;
	QStandardItemModel *modelAux = new QStandardItemModel(0, 19, parent);

	k = 0;
	modelAux->setHeaderData(k++, Qt::Horizontal, QObject::tr("Nmero"), Qt::DisplayRole);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Incio"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Fim"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Tipo"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Codificador"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("IP Origem"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Usurio Origem"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("IP Destino"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Usurio Destino"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Estado"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Durao"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Jitter Mdio Org. [s]"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Jitter Mx. Org. [s]"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Perda Pacotes Org. [%]"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Erro Sequncia Org."), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Jitter Mdio Rcpt. [s]"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Jitter Mx. Rcpt. [s]"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Perda Pacotes Rcpt. [%]"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(k, Qt::Horizontal, QObject::tr("Erro Sequncia Rcpt."), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(k++)->setTextAlignment(Qt::AlignCenter);
	return modelAux;
	}

void WindowReport::setSourceModel(QAbstractItemModel *modelAux)
	{
	transModel->setSourceModel(modelAux);
	}

void WindowReport::setInfoPackets(QList <infoReportTrans_t> infoPacket, QList<packet_t> listP)
	{
	int listaSize, j, k;
	infoReportTrans_t infoAux;

	listaSize = infoPacket.size();
	infoReportAux = infoPacket;

	for (j = 0; j < listaSize; j++)
		{
		infoAux = infoPacket.at(j);

		model->insertRow(j);
		k = 0;
		//Number
		model->setData(model->index(j, k), infoAux.numTrans + 1);
		model->itemFromIndex(model->index(j, k++))->setEditable(false);

		//Start
		model->setData(model->index(j, k), infoAux.tStart);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//End
		model->setData(model->index(j, k), infoAux.tStop);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Type
		model->setData(model->index(j, k), infoAux.Tipo);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		if (infoPacket.at(j).erroSIP == ERROR_SIP_SERIOUS)
			{
			model->itemFromIndex(model->index(j,k))->setBackground(QBrush(QColor("red"), Qt::SolidPattern));
			model->itemFromIndex(model->index(j,k))->setToolTip(tr("Erro de sinalizao de alta gravidade"));
			}
		else if (infoPacket.at(j).erroSIP == ERROR_SIP_MIDDLE)
			{
			model->itemFromIndex(model->index(j,k))->setBackground(QBrush(QColor("yellow"), Qt::SolidPattern));
			model->itemFromIndex(model->index(j,k))->setToolTip(tr("Erro de sinalizao de mdia gravidade"));
			}
		else if (infoPacket.at(j).erroSIP == ERROR_SIP_LIGHT)
			{
			model->itemFromIndex(model->index(j,k))->setBackground(QBrush(QColor("blue"), Qt::SolidPattern));
			model->itemFromIndex(model->index(j,k))->setToolTip(tr("Erro de sinalizao de gravidade leve"));
			}
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Codec
		model->setData(model->index(j, k), infoAux.Codec);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		if (infoPacket.at(j).erroRTP == ERROR_RTP_SERIOUS)
			{
			model->itemFromIndex(model->index(j,k))->setBackground(QBrush(QColor("red"), Qt::SolidPattern));
			model->itemFromIndex(model->index(j,k))->setToolTip(tr("Erro de fluxo RTP de alta gravidade"));
			}
		else if (infoPacket.at(j).erroRTP == ERROR_RTP_MIDDLE)
			{
			model->itemFromIndex(model->index(j,k))->setBackground(QBrush(QColor("yellow"), Qt::SolidPattern));
			model->itemFromIndex(model->index(j,k))->setToolTip(tr("Erro de fluxo RTP de mdia gravidade"));
			}
		else if (infoPacket.at(j).erroRTP == ERROR_RTP_LIGHT)
			{
			model->itemFromIndex(model->index(j,k))->setBackground(QBrush(QColor("blue"), Qt::SolidPattern));
			model->itemFromIndex(model->index(j,k))->setToolTip(tr("Erro de fluxo RTP de gravidade leve"));
			}
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Source IP
		model->setData(model->index(j, k), infoAux.IPFrom);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Source User
		model->setData(model->index(j, k), infoAux.userFrom);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Destination IP
		model->setData(model->index(j, k), infoAux.IPTo);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Destination User
		model->setData(model->index(j, k), infoAux.userTo);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Status
		model->setData(model->index(j, k), infoAux.state);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Duration
		model->setData(model->index(j, k), infoAux.duration);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Mean source jitter
		model->setData(model->index(j, k), infoAux.meanJitterOrg);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Max source jitter
		model->setData(model->index(j, k), infoAux.maxJitterOrg);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Packets lost from source
		model->setData(model->index(j, k), infoAux.packetLostPercOrg);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Sequence errors from source
		model->setData(model->index(j, k), infoAux.sequenceErrorOrg);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Mean receiver jitter
		model->setData(model->index(j, k), infoAux.meanJitterRec);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Max receiver jitter
		model->setData(model->index(j, k), infoAux.maxJitterRec);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Packets lost from receiver
		model->setData(model->index(j, k), infoAux.packetLostPercRec);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);

		//Sequence errors from receiver
		model->setData(model->index(j, k), infoAux.sequenceErrorRec);
		model->itemFromIndex(model->index(j,k))->setEditable(false);
		model->itemFromIndex(model->index(j,k++))->setTextAlignment(Qt::AlignCenter);		
		}
	}

void WindowReport::tClicked(QModelIndex modelIndex)
	{
	infoReportTrans_t infoAux;

	infoAux = infoReportAux.at(modelIndex.row());
	infoView->clear();
	if (infoAux.erroSIPText.isEmpty() == false)
		{
		infoView->append(infoAux.erroSIPText);
		}
	if (infoAux.erroRTPText.isEmpty() == false)
		{
		infoView->append(infoAux.erroRTPText);
		}
	}
