#include <QtGui>
#include "windowtransactions.h"


WindowTransactions::WindowTransactions()
	{
	setupUi(this);

	setMinimumSize(900, 600);
	setWindowTitle(tr("Transaes"));
	setAttribute(Qt::WA_DeleteOnClose);

	creatActions();
	packetsModel = new QSortFilterProxyModel;
	packetsModel->setDynamicSortFilter(true);

	model = createMailModel(this);
	setSourceModel(model);

	packetsView->setMinimumSize(900, 600);
	packetsView->setRootIsDecorated(false);
	packetsView->setAlternatingRowColors(true);
	packetsView->setModel(packetsModel);
	packetsView->setSortingEnabled(false);
	packetsView->sortByColumn(0, Qt::AscendingOrder);
	packetsView->setSelectionMode(QAbstractItemView::ExtendedSelection);
	packetsView->setSelectionBehavior(QAbstractItemView::SelectRows);
	packetsView->setFont(QFont("Courier New"));

	packetsView->setColumnWidth(0, 55);		//Number
	packetsView->setColumnWidth(1, 120);	//Start
	packetsView->setColumnWidth(2, 70);		//Type
	packetsView->setColumnWidth(3, 120);	//Source IP
	packetsView->setColumnWidth(4, 140);	//Source User
	packetsView->setColumnWidth(5, 120);	//Desination IP
	packetsView->setColumnWidth(6, 140);	//Destination User
	packetsView->setColumnWidth(7, 240);	//Status

	connect (packetsView, SIGNAL(clicked(QModelIndex)), this, SLOT(tClicked(QModelIndex)));
	numTransactions = 0;	
	}

WindowTransactions::~WindowTransactions(void)
   {
   delete packetsModel;
   delete model;
   delete graphAct;
   delete packetAct;
   delete reportAct;
   delete playAct;
   delete filterAct;
   }

void WindowTransactions::creatActions(void)
	{
	graphAct = new QAction(tr("&Grfico"), this);
	graphAct->setStatusTip(tr("Exibe grfico da transao"));
	connect(graphAct, SIGNAL(triggered()), this, SLOT(graphShow()));

	packetAct = new QAction(tr("&Pacotes"), this);
	packetAct->setStatusTip(tr("Exibe pacotes da transao"));
	connect(packetAct, SIGNAL(triggered()), this, SLOT(pacoteShow()));

	reportAct = new QAction(tr("&Relatrio"), this);
	reportAct->setStatusTip(tr("Exibe relatrios da transao"));
	connect(reportAct, SIGNAL(triggered()), this, SLOT(reportShow()));

	playAct = new QAction(tr("&Reproduzir udio"), this);
	playAct->setStatusTip(tr("Reproduz o udio da chamada"));
	connect(playAct, SIGNAL(triggered()), this, SLOT(playExibe()));

	filterAct = new QAction(tr("&Filtrar Transaes"), this);
	filterAct->setStatusTip(tr("Filtrar Transaes"));
	connect(filterAct, SIGNAL(triggered()), this, SIGNAL(transFilter()));
	}

void WindowTransactions::graphShow(void)
	{
	QList<int> transSel;
	QItemSelectionModel* ism = packetsView->selectionModel();
	QModelIndexList indexList = ism->selectedRows();
	QModelIndex index;
	int listaSize;

	if ((listaSize = indexList.size()))
		{
		transSel.clear();
		for (int i = 0; i < listaSize; i++)
			{
			index = indexList.at(i);			
			transSel.append(model->item(index.row(), 0)->text().toInt() - 1);
			}
		emit (transSelected(transSel));
		}
	else
		{
		//Show error message
		}
	}

void WindowTransactions::pacoteShow(void)
	{
	QList<int> transSel;
	QItemSelectionModel* ism = packetsView->selectionModel();
	QModelIndexList indexList = ism->selectedRows();
	QModelIndex index;
	int listSize;

	if ((listSize = indexList.size()))
		{
		transSel.clear();
		for (int i = 0; i < listSize; i++)
			{
			index = indexList.at(i);
			transSel.append(model->item(index.row(), 0)->text().toInt() - 1);
			}
      emit (transSelectedLastPacket(transSel));
		}
	else
		{
		//Show error message
		}
	}

void WindowTransactions::reportShow(void)
	{
	QList<int> transSel;
	QItemSelectionModel* ism = packetsView->selectionModel();
	QModelIndexList indexList = ism->selectedRows();
	QModelIndex index;
	int listSize;

	if ((listSize = indexList.size()))
		{
		transSel.clear();
		for (int i = 0; i < listSize; i++)
			{
			index = indexList.at(i);
			transSel.append(model->item(index.row(), 0)->text().toInt() - 1);
			}
		emit (transSelectedReport(transSel));
		}
	else
		{
		//Show error message
		}
	}

void WindowTransactions::playExibe(void)
	{
	QList<int> transSel;
	QItemSelectionModel* ism = packetsView->selectionModel();
	QModelIndexList indexList = ism->selectedRows();
	QModelIndex index;
	int listSize;

	if ((listSize = indexList.size()))
		{
		transSel.clear();
		for (int i = 0; i < listSize; i++)
			{
			index = indexList.at(i);
			transSel.append(model->item(index.row(), 0)->text().toInt() - 1);
			}
		emit (transSelectedPlay(transSel.at(0)));
		}
	else
		{
		//Show error message
		}
	}

void WindowTransactions::contextMenuEvent(QContextMenuEvent *event)
	{
	QMenu menu(this);
	menu.addAction(graphAct);
	menu.addAction(packetAct);
	menu.addAction(reportAct);
	menu.addAction(playAct);
//	menu.addAction(filterAct);
	menu.exec(event->globalPos());
	}

void WindowTransactions::tClicked(QModelIndex modelIndex)
	{
	emit (transClicked(model->item(modelIndex.row(), 0)->text().toInt() - 1));
	}

QStandardItemModel * WindowTransactions::createMailModel(QObject *parent)
	{
   QStandardItemModel *modelAux = new QStandardItemModel(0, 8, parent);

   modelAux->setHeaderData(0, Qt::Horizontal, QObject::tr("Nmero"), Qt::DisplayRole);

   modelAux->setHeaderData(1, Qt::Horizontal, QObject::tr("Incio"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(1)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(2, Qt::Horizontal, QObject::tr("Tipo"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(2)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(3, Qt::Horizontal, QObject::tr("IP Origem"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(3)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(4, Qt::Horizontal, QObject::tr("Usurio Origem"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(4)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(5, Qt::Horizontal, QObject::tr("IP Destino"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(5)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(6, Qt::Horizontal, QObject::tr("Usurio Destino"), Qt::DisplayRole);
	modelAux->horizontalHeaderItem(6)->setTextAlignment(Qt::AlignCenter);

	modelAux->setHeaderData(7, Qt::Horizontal, QObject::tr("Estado"), Qt::DisplayRole);

	return modelAux;
	}

void WindowTransactions::setSourceModel(QAbstractItemModel *modelAux)
	{
	packetsModel->setSourceModel(modelAux);
	}

void WindowTransactions::addPacket(int numTrans, char * tStart, char * tStop, char * Type, char * IPFrom, char * userFrom, char * IPTo, char * userTo)
	{
	model->insertRow(numTransactions);

	model->setData(model->index(numTransactions, 0), numTrans+1);
	model->itemFromIndex(model->index(numTransactions,0))->setEditable(false);

   model->setData(model->index(numTransactions, 1), tStart);
	model->itemFromIndex(model->index(numTransactions,1))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,1))->setTextAlignment(Qt::AlignCenter);

	if (Type == NULL)
		{
		model->setData(model->index(numTransactions, 2), tr("Desconhecido"));
		}
	else
		{
		model->setData(model->index(numTransactions, 2), Type);
		}
	model->itemFromIndex(model->index(numTransactions,2))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,2))->setTextAlignment(Qt::AlignCenter);

   model->setData(model->index(numTransactions, 3), IPFrom);
	model->itemFromIndex(model->index(numTransactions,3))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,3))->setTextAlignment(Qt::AlignCenter);

   model->setData(model->index(numTransactions, 4), userFrom);
	model->itemFromIndex(model->index(numTransactions,4))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,4))->setTextAlignment(Qt::AlignCenter);

   model->setData(model->index(numTransactions, 5), IPTo);
	model->itemFromIndex(model->index(numTransactions,5))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,5))->setTextAlignment(Qt::AlignCenter);

   model->setData(model->index(numTransactions, 6), userTo);
	model->itemFromIndex(model->index(numTransactions,6))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,6))->setTextAlignment(Qt::AlignCenter);

	model->setData(model->index(numTransactions, 7), tr("Iniciada"));
	model->itemFromIndex(model->index(numTransactions,7))->setEditable(false);

	numTransactions++;
	}

void WindowTransactions::addPacketComplete(int numTrans, char * tStart, char * tStop, char * Type, char * IPFrom, char * userFrom, char * IPTo, char * userTo, char * State)
	{
	model->insertRow(numTransactions);

	model->setData(model->index(numTransactions, 0), numTrans+1);
	model->itemFromIndex(model->index(numTransactions,0))->setEditable(false);

	model->setData(model->index(numTransactions, 1), tStart);
	model->itemFromIndex(model->index(numTransactions,1))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,1))->setTextAlignment(Qt::AlignCenter);

	model->setData(model->index(numTransactions, 2), Type);
	model->itemFromIndex(model->index(numTransactions,2))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,2))->setTextAlignment(Qt::AlignCenter);

	model->setData(model->index(numTransactions, 3), IPFrom);
	model->itemFromIndex(model->index(numTransactions,3))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,3))->setTextAlignment(Qt::AlignCenter);

	model->setData(model->index(numTransactions, 4), userFrom);
	model->itemFromIndex(model->index(numTransactions,4))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,4))->setTextAlignment(Qt::AlignCenter);

	model->setData(model->index(numTransactions, 5), IPTo);
	model->itemFromIndex(model->index(numTransactions,5))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,5))->setTextAlignment(Qt::AlignCenter);

	model->setData(model->index(numTransactions, 6), userTo);
	model->itemFromIndex(model->index(numTransactions,6))->setEditable(false);
	model->itemFromIndex(model->index(numTransactions,6))->setTextAlignment(Qt::AlignCenter);

	model->setData(model->index(numTransactions, 7), State);
	model->itemFromIndex(model->index(numTransactions,7))->setEditable(false);

	numTransactions++;
	}

void WindowTransactions::rxUpdateTransaction (int numTransaction, char * newState)
   {
   if (numTransaction < numTransactions)
      model->item(numTransaction, 7)->setText(newState);
   }

void WindowTransactions::closeEvent(QCloseEvent *event)
   {
   emit closingTrans(true);
   }

void WindowTransactions::clearTransactions(void)
   {
   model->clear();
   model = createMailModel(this);
   setSourceModel(model);
   numTransactions = 0;
	packetsView->setColumnWidth(0, 55);		//Number
	packetsView->setColumnWidth(1, 120);	//Start
	packetsView->setColumnWidth(2, 70);		//Type
	packetsView->setColumnWidth(3, 120);	//Source IP
	packetsView->setColumnWidth(4, 140);	//Source User
	packetsView->setColumnWidth(5, 120);	//Destination IP
	packetsView->setColumnWidth(6, 140);	//Destination User
	packetsView->setColumnWidth(7, 240);	//Status
	}



